@@ -11,7 +11,7 @@ if not modules then modules = { } end modules['mtx-context'] = {
 
 local type, next, tostring, tonumber = type, next, tostring, tonumber
 local format, gmatch, match, gsub, find = string.format, string.gmatch, string.match, string.gsub, string.find
-local quote, validstring, splitstring = string.quote, string.valid, string.split
+local quote, validstring, splitstring, splitlines = string.quote, string.valid, string.split, string.splitlines
 local sort, concat, insert, sortedhash, tohash = table.sort, table.concat, table.insert, table.sortedhash, table.tohash
 local settings_to_array = utilities.parsers.settings_to_array
 local appendtable = table.append
@@ -33,9 +33,14 @@ local removefile    = os.remove
 local renamefile    = os.rename
 local formatters    = string.formatters
 
+local starttiming   = statistics.starttiming
+local stoptiming    = statistics.stoptiming
+local resettiming   = statistics.resettiming
+local elapsedtime   = statistics.elapsedtime
+
 local application = logs.application {
     name     = "mtx-context",
-    banner   = "ConTeXt Process Management 1.04",
+    banner   = "ConTeXt Process Management 1.06",
  -- helpinfo = helpinfo, -- table with { category_a = text_1, category_b = text_2 } or helpstring or xml_blob
     helpinfo = "mtx-context.xml",
 }
@@ -114,7 +119,7 @@ local function restart(engine_old,engine_new)
     local generate  = environment.arguments.generate and (engine_new == "luatex" or engine_new == "luajittex")
     local arguments = generate and  "--generate" or environment.reconstructcommandline()
     local ownname   = filejoinname(filepathpart(environment.ownname),"mtxrun.lua")
-    local command   = format("%s --luaonly %q %s --redirected",engine_new,ownname,arguments)
+    local command   = format("%s --luaonly --socket %q %s --redirected",engine_new,ownname,arguments)
     report(format("redirect %s -> %s: %s",engine_old,engine_new,command))
     local result = os.execute(command)
     os.exit(result == 0 and 0 or 1)
@@ -207,7 +212,9 @@ local temporary_suffixes = {
 }
 
 local synctex_runfiles = {
-    "synctex", "synctex.gz", "syncctx" -- synctex
+    "synctex",    -- should not be created by engine
+    "synctex.gz", -- not used
+ -- "syncctx"     --
 }
 
 local persistent_runfiles = {
@@ -470,7 +477,7 @@ end
 local pdfview -- delayed
 
 local function pdf_open(name,method)
-    statistics.starttiming("pdfview")
+    starttiming("pdfview")
     pdfview = pdfview or dofile(resolvers.findfile("l-pdfview.lua","tex"))
     pdfview.setmethod(method)
     report(pdfview.status())
@@ -479,12 +486,12 @@ local function pdf_open(name,method)
         pdfname = name .. ".pdf" -- agressive
     end
     pdfview.open(pdfname)
-    statistics.stoptiming("pdfview")
-    report("pdfview overhead: %s seconds",statistics.elapsedtime("pdfview"))
+    stoptiming("pdfview")
+    report("pdfview overhead: %s seconds",elapsedtime("pdfview"))
 end
 
 local function pdf_close(name,method)
-    statistics.starttiming("pdfview")
+    starttiming("pdfview")
     pdfview = pdfview or dofile(resolvers.findfile("l-pdfview.lua","tex"))
     pdfview.setmethod(method)
     local pdfname = filenewsuffix(name,"pdf")
@@ -493,7 +500,7 @@ local function pdf_close(name,method)
     end
     pdfname = name .. ".pdf" -- agressive
     pdfview.close(pdfname)
-    statistics.stoptiming("pdfview")
+    stoptiming("pdfview")
 end
 
 -- result file handling
@@ -611,17 +618,20 @@ local function flags_to_string(flags,prefix)
     -- context flags get prepended by c: ... this will move to the sbx module
     local t = { }
     for k, v in table.sortedhash(flags) do
+        local p
         if prefix then
-            k = format("c:%s",k)
+            p = format("c:%s",k)
+        else
+            p = k
         end
         if not v or v == "" or v == '""' then
             -- no need to flag false
         elseif v == true then
-            t[#t+1] = format('--%s',k)
+            t[#t+1] = format('--%s',p)
         elseif type(v) == "string" then
-            t[#t+1] = format('--%s=%s',k,quote(v))
+            t[#t+1] = format('--%s=%s',p,quote(v))
         else
-            t[#t+1] = format('--%s=%s',k,tostring(v))
+            t[#t+1] = format('--%s=%s',p,tostring(v))
         end
     end
     return concat(t," ")
@@ -694,6 +704,7 @@ function scripts.context.run(ctxdata,filename)
     local a_jithash       = getargument("jithash")
     local a_permitloadlib = getargument("permitloadlib")
     local a_texformat     = getargument("texformat")
+    local a_notuc         = getargument("notuc")
     local a_keeptuc       = getargument("keeptuc")
     local a_keeplog       = getargument("keeplog")
     local a_keeppdf       = getargument("keeppdf")
@@ -893,7 +904,7 @@ function scripts.context.run(ctxdata,filename)
                 --
                 local l_flags = {
                     ["interaction"]           = a_batchmode,
-                 -- ["synctex"]               = false,       -- context has its own way
+                    ["synctex"]               = false,       -- context has its own way
                  -- ["no-parse-first-line"]   = true,        -- obsolete
                  -- ["safer"]                 = a_safer,     -- better use --sandbox
                  -- ["no-mktex"]              = true,
@@ -908,6 +919,8 @@ function scripts.context.run(ctxdata,filename)
                 --
                 local directives = { }
                 --
+                -- todo: handle these at the tex end
+                --
                 if a_nodates then
                     directives[#directives+1] = format("backend.date=%s",type(a_nodates) == "string" and a_nodates or "no")
                 end
@@ -920,6 +933,11 @@ function scripts.context.run(ctxdata,filename)
                     directives[#directives+1] = format("system.profile=%s",tonumber(a_profile) or 0)
                 end
                 --
+             -- if a_notuc then
+             --     removefile(fileaddsuffix(jobname,"tuc"))
+             --     directives[#directives+1] = "job.save=no" -- handled at tex end
+             -- end
+                --
                 for i=1,#synctex_runfiles do
                     removefile(fileaddsuffix(jobname,synctex_runfiles[i]))
                 end
@@ -934,6 +952,15 @@ function scripts.context.run(ctxdata,filename)
                 --
                 multipass_copypdffile(jobname,a_keeppdf or analysis.keeppdf)
                 --
+                local signal = analysis.signal or environment.arguments.signal
+                if type(signal) == "string" then
+                    signalled = utilities.signals.initialize(signal)
+                end
+                --
+                if signalled then
+                    signalled("reset",0)
+                end
+                --
                 for currentrun=1,maxnofruns do
                     --
                     c_flags.final      = false
@@ -943,9 +970,14 @@ function scripts.context.run(ctxdata,filename)
                     c_flags.currentrun = currentrun
                     c_flags.noarrange  = a_noarrange or a_arrange or nil
                     c_flags.profile    = a_profile and (tonumber(a_profile) or 0) or nil
+                    c_flags.signal     = signal
                     --
                     print("") -- cleaner, else continuation on same line
                     --
+                    if signalled then
+                        signalled("busy",currentrun)
+                    end
+                    --
                     local returncode = environment.run_format(
                         formatfile,
                         scriptfile,
@@ -961,8 +993,11 @@ function scripts.context.run(ctxdata,filename)
                             result_save_error(oldbase,newbase)
                         end
                         os.exit(1)
+                        if signalled then
+                            signalled("error",currentrun)
+                        end
                         break
-                    elseif returncode == 0 then
+                    elseif returncode == 0 or signal == "qr" then
                         multipass_copyluafile(jobname,a_keeptuc and currentrun)
                         multipass_copylogfile(jobname,a_keeplog and currentrun)
                         if not multipass_forcedruns then
@@ -970,15 +1005,27 @@ function scripts.context.run(ctxdata,filename)
                             if multipass_changed(oldhash,newhash) then
                                 changed[jobname] = true
                                 oldhash = newhash
+                                if signalled then
+                                    signalled(currentrun == maxnofruns and "maxruns" or "done",currentrun)
+                                end
                             else
+                                if signalled then
+                                    signalled("finished",currentrun)
+                                end
                                 break
                             end
                         elseif currentrun == multipass_forcedruns then
+                            if signalled then
+                                signalled("maxruns",currentrun)
+                            end
                             report("quitting after force %i runs",multipass_forcedruns)
                             break
                         end
                     else
                         report("fatal error: return code: %s",returncode or "?")
+                        if signalled then
+                            signalled("error",currentrun)
+                        end
                         if resultname then
                             result_save_error(oldbase,newbase)
                         end
@@ -995,15 +1042,10 @@ function scripts.context.run(ctxdata,filename)
                         io.savedata(logfile,(gsub(logdata,"%[.-m","")))
                     end
                 end
-                --
-                --
-                --  this will go away after we update luatex
-                --
-                local syncctx = fileaddsuffix(jobname,"syncctx")
-                if validfile(syncctx) then
-                    renamefile(syncctx,fileaddsuffix(jobname,"synctex"))
-                end
-                --
+             -- local syncctx = fileaddsuffix(jobname,"syncctx")
+             -- if validfile(syncctx) then
+             --     renamefile(syncctx,fileaddsuffix(jobname,"synctex"))
+             -- end
                 if a_arrange then
                     --
                     c_flags.final      = true
@@ -1205,10 +1247,10 @@ do -- more or less copied from mtx-plain.lua:
     local function engine(texengine,texformat)
         local command = string.format('%s --ini --etex --8bit %s \\dump',texengine,fileaddsuffix(texformat,"mkii"))
         if environment.arguments.silent then
-            statistics.starttiming()
+            starttiming()
             local command = format("%s > temp.log",command)
             local result  = os.execute(command)
-            local runtime = statistics.stoptiming()
+            local runtime = stoptiming()
             if result ~= 0 then
                 print(format("%s silent make > fatal error when making format %q",texengine,texformat)) -- we use a basic print
             else
@@ -1247,7 +1289,7 @@ do -- more or less copied from mtx-plain.lua:
                 fmtpathspec = "."
             end
         end
-        fmtpathspec = string.splitlines(fmtpathspec)[1] or fmtpathspec
+        fmtpathspec = splitlines(fmtpathspec)[1] or fmtpathspec
         fmtpathspec = fmtpathspec and file.splitpath(fmtpathspec)
         local fmtpath = nil
         if fmtpathspec then
@@ -1384,6 +1426,36 @@ function scripts.context.version()
     end
 end
 
+-- todo: also check size of format, should not be excessive
+
+function scripts.context.integrity()
+    local filename = "luametatex.h"
+    local fullname = resolvers.findfile(filename) or ""
+    local data     = fullname ~= "" and io.loaddata(fullname) or ""
+    local mismatch = false
+    report("reference source file : %s",filename)
+    local version, development_id = match(data,
+         [[# *define *luametatex_version_string *"([%d%.]+)"%s]]
+      .. [[# *define *luametatex_development_id *(%d+)%s]]
+    )
+    if tostring(version) ~= tostring(status.version) then
+        mismatch = true
+    elseif tostring(development_id) ~= tostring(status.development_id) then
+        mismatch = true
+    end
+    report()
+    report("binary version        : %s",status.version)
+    report("source version        : %s",version or "unknown")
+    report("binary development id : %s",status.development_id)
+    report("source development id : %s",development_id or "unknown")
+    report()
+    if data == "" then
+    report("problem encountered   : the source file is missing (incomplete installation)")
+    elseif mismatch then
+    report("problem encountered   : there is a mismatch between source and binary")
+    end
+end
+
 function scripts.context.purge_job(jobname,all,mkiitoo,fulljobname)
     if jobname and jobname ~= "" then
         jobname = filebasename(jobname)
@@ -1726,6 +1798,324 @@ function scripts.context.extra()
     end
 end
 
+-- experiment
+
+do
+
+    local popen   = io.popen
+    local close   = io.close
+    ----- read    = io.read
+    local gobble  = io.gobble or function(f) f:read("l") end
+    ----- clock   = os.clock
+    local ticks   = lua.getpreciseticks
+    local seconds = lua.getpreciseseconds
+
+    local f_runner  = formatters['context %s "%s"']
+    local f_command = formatters['%s %s']
+
+    local function getline(line,name)
+        if string.find(line,"^context ") then
+            return line
+        elseif string.find(line,"^mtxrun ") then
+            return line
+        end
+    end
+
+    function scripts.context.parallel()
+        if getargument("pattern") then
+            environment.files = dir.glob(getargument("pattern"))
+        end
+        local files    = environment.files
+        local total    = files and #files or 0
+        local lists    = nil
+        local whattodo = "filename"
+        local terminal = environment.argument("terminal")
+        local parallel = environment.argument("parallel")
+        local runners  = tonumber(parallel) or 8
+
+        if type(parallel) == "string" then
+            local specification = table.load(parallel)
+            if specification then
+                local parallel = specification.parallel
+                if parallel then
+                    specification = parallel
+                end
+                runners  = tonumber(specification.runners) or runners
+                terminal = specification.terminal or terminal
+                sequence = specification.sequence
+                commands = specification.commands
+                if commands then
+                    if sequence then
+                        lists = commands
+                        for i=1,#commands do
+                            total = total + #commands[i]
+                        end
+                    else
+                        local list = commands[1]
+                        for i=2,#commands do
+                            local c = commands[i]
+                            for i=1,#c do
+                                list[#list+1] = c[i]
+                            end
+                            total = total + #c
+                        end
+                        lists = { list }
+                    end
+                    whattodo = "command"
+                else
+                    report("missing commands list in %a",parallel)
+                    return
+                end
+            else
+                report("invalid specification %a",parallel)
+                return
+            end
+        elseif getargument("parallelsequence") then
+            total = 0
+            for i=1,#files do
+                local list = { }
+                local name = files[i]
+                local data = splitlines(io.loaddata(name) or "")
+                for i=1,#data do
+                    local line = getline(data[i],name)
+                    if line then
+                        list[#list+1] = line
+                    end
+                end
+                if #list > 0 then
+                    if not lists then
+                        lists = { list, filename = name }
+                    else
+                        lists[#lists+1] = list
+                    end
+                    total = total + #list
+                end
+            end
+            whattodo = "command"
+        elseif getargument("parallellist") then
+            total = 0
+            local list = { }
+            for i=1,#files do
+                local name = files[i]
+                local data = splitlines(io.loaddata(name) or "")
+                for i=1,#data do
+                    local line = getline(data[i],name)
+                    if line then
+                        list[#list+1] = line
+                    end
+                end
+            end
+            if #list > 0 then
+                lists = { list }
+                total = total + #list
+            end
+            whattodo = "command"
+        elseif total > 1 then
+            lists = { files }
+        end
+
+        if not lists and total == 1 then
+            -- Beware: "--parallel" and "--terminal" are passed to the single run but this
+            -- is normally harmless.
+            scripts.context.autoctx()
+        elseif total > 0 then
+            local allresults = { }
+            local start      = starttiming("parallel")
+            local counts     = 0
+            local totals     = 0
+            local problem    = false
+--             local squid      = environment.arguments.squid and require("util-sig-imp-segment.lua")
+local squid      = environment.arguments.squid and require("util-sig-imp-parallel.lua")
+-- if type(signal) == "string" then
+--     signalled = utilities.signals.initialize(signal)
+-- end
+-- if signalled then
+--     signalled("reset",0)
+-- end
+            -- a hack
+            local passthese = environment.arguments_after
+            for i=1,#passthese do
+                local a = passthese[i]
+                if string.find(a,"^%-%-parallel") or string.find(a,"^%-%-terminal") or not find(a,"^%-%-") then
+                    passthese[i] = ""
+                end
+            end
+            passthese = table.unique(passthese)
+            -- end of hack
+if squid then
+    squid.stepper("reset")
+--     squid.signal("reset")
+--     for i=1,runners do
+--         squid.stepper("busy",i)
+--     end
+end
+            --
+            local arguments = environment.reconstructcommandline(passthese)
+            for set=1,#lists do
+                local files   = lists[set]
+                local process = { }
+                local results = { }
+                local count   = 0
+                local total   = #files
+--                 local steps   = { }
+                totals = totals + total
+                allresults[set] = results
+--                 for i=1,runners do
+--                     steps[i] = 0
+--                 end
+                while true do
+                    local done = false
+                    for i=1,runners do
+                        local pi = process[i]
+                        if pi then
+                            local s
+                            if terminal then
+                                s = pi.handle:read("l")
+                                if s then
+                                    done = true
+                                    report("%02i : %s",i,s)
+                                    goto done
+                                end
+                            else
+                                s = gobble(pi.handle)
+                                if s then
+                                    done = true
+                                    goto done
+                                end
+                            end
+                            if not s then
+                                local r, detail, n = close(pi.handle)
+                                stoptiming(pi.timer)
+                                pi.result = (not r or n > 0) and "error" or "done"
+                                pi.time   = elapsedtime(pi.timer)
+                                pi.handle = nil
+                                pi.timer  = nil
+                                if terminal then
+                                    report()
+                                end
+                                report("process %02i, index %02i, %s %a, status %a, runtime %0.3f",i,pi.count,whattodo,pi.filename,pi.result,pi.time)
+                                if terminal then
+                                    report()
+                                end
+                                process[i] = false
+                                results[pi.count] = pi
+if squid and not problem and pi.result == "error"  then
+    problem = true
+    squid.stepper("error") -- ,i,steps[i],true)
+    for i=1,runners do
+        if process[i] then
+--             squid.stepper("busy",i,steps[i],true)
+--             squid.stepper("step",i,steps[i],true)
+            squid.stepper("step",i,0,true)
+        end
+    end
+end
+                            end
+                        end
+                        count  = count + 1
+                        counts = counts + 1
+                        if count > total then
+                            -- we're done
+                        else
+                            local timer    = "parallel:" .. set .. ":" .. i
+                            local filename = files[count]
+                            local dirname  = file.dirname(filename)
+                            local basename = file.basename(filename)
+                            if dirname ~= "." and dirname ~= "./" then
+                                dir.push(dirname)
+                            end
+                            local command  = whattodo == "command" and f_command(basename,arguments) or f_runner(arguments,basename)
+                            resettiming(timer)
+                            starttiming(timer)
+-- steps[i] = steps[i] + 1
+if squid then
+--     squid.stepper("step",i,steps[i],problem)
+-- print(timer)
+    squid.stepper("step",i,0,problem)
+-- if i == 8 then
+--     os.exit()
+-- end
+end
+                            local result  = popen(command)
+                            if dirname ~= "." then
+                                dir.pop()
+                            end
+                            local status  = nil
+                            if result then
+                                process[i] = {
+                                    handle   = result,
+                                    result   = "start",
+                                    filename = filename,
+                                    count    = count,
+                                    time     = 0,
+                                    timer    = timer,
+                                }
+                                status = process[i]
+                            else
+                                status = {
+                                    result   = "error",
+                                    count    = count,
+                                    filename = filename,
+                                    time     = 0,
+                                }
+                                problem = true
+                                stoptiming(timer)
+                            end
+                            results[count] = status
+                            if terminal then
+                                report()
+                            end
+                            report("process %02i, index %02i, %s %a, status %a",i,status.count,whattodo,status.filename,status.result)
+                            if terminal then
+                                report()
+                            end
+                            done = true
+                        end
+                      ::done::
+                    end
+                    if not done then
+                        break
+                    end
+                end
+            end
+            stoptiming("parallel")
+            report()
+            report("files: %i, runtime: %s",totals,elapsedtime("parallel"))
+            report()
+            local errors = { }
+            for set=1,#allresults do
+                if set > 1 then
+                    report()
+                end
+                local results = allresults[set]
+                local total   = #results
+                for i=1,total do
+                    local ri = results[i]
+                    local result   = ri.result
+                    local filename = ri.filename
+                    if result == "error" then
+                        errors[#errors+1] = filename
+                    end
+                    report("index %02i, %s %a, status %a, runtime %0.3f ",ri.count,whattodo,filename,result,ri.time)
+                end
+            end
+            if squid then
+                squid.stepper((problem or #errors > 0) and "error" or "finished")
+            end
+            if #errors > 0 then
+                report()
+                report("errors in:")
+                report()
+                for i=1,#errors do
+                    report("  %s",errors[i])
+                end
+            end
+            report()
+        end
+    end
+
+end
+
 -- todo: we need to do a dummy run
 
 local function showsetter()
@@ -1746,10 +2136,55 @@ function scripts.context.logcategories()
     scripts.context.run()
 end
 
+function scripts.context.find(pattern)
+    if pattern ~= "" then
+        local found = resolvers.findfile("luametatex.tex")
+        if found and found ~= "" then
+            for i=1,6 do
+                found = file.pathpart(found)
+            end
+        end
+        local files = dir.glob(found.."**.tex")
+        for i=1,#files do
+            local f = files[i]
+            local d = io.loaddata(f)
+            if find(d,pattern) then
+                local l = splitlines(d)
+                report(f)
+                report("")
+                for i=1,#l do
+                    if find(l[i],pattern) then
+                        report("%5i  %s",i,l[i])
+                    end
+                end
+                report("")
+            end
+        end
+    else
+        report("there is nothing to search for")
+    end
+end
+
 function scripts.context.timed(action)
     statistics.timed(action,true)
 end
 
+function scripts.context.runlua(filename)
+    if filename and filesuffix(filename) == "lua" then
+        local chunk = io.loadchunk(filename) -- 1024
+        if chunk and find(chunk,"^%-%- +runner=mtxrun") then
+            local result = os.execute("luametatex --luaonly " .. filename)
+            os.exit(result)
+            return true
+        end
+    end
+    return false
+end
+
+-- We don't want this as we might want to expand later on:
+
+-- environment.files = environment.globfiles()
+
 -- getting it done
 
 if getargument("pdftex") then
@@ -1815,7 +2250,9 @@ else
     multipass_forcedruns = tonumber(getargument("forcedruns")) or nil
 end
 
-if getargument("run") then
+if getargument("parallel") or getargument("parallellist") then
+    scripts.context.timed(scripts.context.parallel)
+elseif getargument("run") then
     scripts.context.timed(scripts.context.autoctx)
 elseif getargument("make") then
     scripts.context.timed(function() scripts.context.make() end)
@@ -1826,11 +2263,14 @@ elseif getargument("ctx") and not getargument("noctx") then
 elseif getargument("version") then
     application.identify()
     scripts.context.version()
+elseif getargument("integrity") then
+    application.identify()
+    scripts.context.integrity()
 elseif getargument("touch") then
     scripts.context.touch()
 elseif getargument("pages") then
     scripts.context.pages()
-elseif getargument("expert") then
+elseif getargument("expert") then -- or getargument("special") then
     application.help("expert", "special")
 elseif getargument("showmodules") or getargument("modules") then
     scripts.context.modules()
@@ -1854,7 +2294,20 @@ elseif getargument("showdirectives") or getargument("directives") == true then
 elseif getargument("showlogcategories") then
     scripts.context.logcategories()
 elseif environment.filenames[1] or getargument("nofile") then
-    scripts.context.timed(scripts.context.autoctx)
+ -- --
+ -- -- How compatible is this ... we might want to resolve the wildcard at the TeX, so
+ -- -- we just keep this as unsuported feature (but at least we know of this case):
+ --
+ -- if not getargument("pattern") and find(environment.filenames[1],"%*") then
+ --     environment.filenames = dir.glob(environment.filenames[1])
+ --  -- setargument("pattern",dir.glob(environment.filenames[1]))
+ -- end
+ --
+    if scripts.context.runlua(environment.filenames[1]) then
+        os.exit()
+    else
+        scripts.context.timed(scripts.context.autoctx)
+    end
 elseif getargument("pipe") then
     scripts.context.timed(scripts.context.pipe)
 elseif getargument("purge") then
@@ -1863,6 +2316,9 @@ elseif getargument("purge") then
 elseif getargument("purgeall") then
     -- only when no filename given, supports --pattern
     scripts.context.purge(true,nil,true)
+elseif getargument("find") then
+    -- context --find="%\framed"
+    scripts.context.find(getargument("find"))
 elseif getargument("pattern") then
     environment.filenames = dir.glob(getargument("pattern"))
     scripts.context.timed(scripts.context.autoctx)
