@@ -16,7 +16,12 @@ local lower, gsub = string.lower, string.gsub
 local concat = table.concat
 local write_nl = (logs and logs.writer) or (texio and texio.write_nl) or print
 
-local otlversion  = 3.133
+local versions = {
+    otl = 3.145,
+    one = 1.541,
+    afm = 1.541,
+    pfb = 1.003,
+}
 
 local helpinfo = [[
 <?xml version="1.0"?>
@@ -24,7 +29,7 @@ local helpinfo = [[
  <metadata>
   <entry name="name">mtx-fonts</entry>
   <entry name="detail">ConTeXt Font Database Management</entry>
-  <entry name="version">1.00</entry>
+  <entry name="version">1.21</entry>
  </metadata>
  <flags>
   <category name="basic">
@@ -51,6 +56,10 @@ local helpinfo = [[
     <flag name="names"><short>use name instead of unicodes</short></flag>
     <flag name="cache" value="str"><short>use specific cache (otl or otf)</short></flag>
    </subcategory>
+   <subcategory>
+    <flag name="pattern" value="str"><short>filter files using pattern</short></flag>
+    <flag name="coverage" value="str"><short>character list</short></flag>
+   </subcategory>
   </category>
  </flags>
  <examples>
@@ -85,6 +94,10 @@ local helpinfo = [[
     <example><command>mtxrun --script font --convert texgyrepagella-regular.otf</command></example>
     <example><command>mtxrun --script font --convert --names texgyrepagella-regular.otf</command></example>
    </subcategory>
+   <subcategory>
+    <example><command>mtxrun --script font --coverage="U+123 U+124" --pattern=texgyre*</command></example>
+    <example><command>mtxrun --script font --coverage="✓"</command></example>
+   </subcategory>
   </category>
  </examples>
 </application>
@@ -92,7 +105,7 @@ local helpinfo = [[
 
 local application = logs.application {
     name     = "mtx-fonts",
-    banner   = "ConTeXt Font Database Management 0.21",
+    banner   = "ConTeXt Font Database Management 1.22",
     helpinfo = helpinfo,
 }
 
@@ -288,14 +301,19 @@ local function showfeatures(tag,specification)
                 for f,ff in table.sortedhash(data) do
                     local done = false
                     for s, ss in table.sortedhash(ff) do
-                        if s == "*"  then s       = "all" end
-                        if ss  ["*"] then ss["*"] = nil ss.all = true end
+                        local s = s == "*" and all or s
+                        if ss["*"] then
+                            ss["*"] = nil
+                            ss.all  = true
+                        end
+                        local name
                         if done then
-                            f = ""
+                            name = ""
                         else
                             done = true
+                            name = f
                         end
-                        report("  %-8s %-8s %-8s",f,s,concat(table.sortedkeys(ss), " ")) -- todo: padd 4
+                        report("  %-8s %-8s %-8s",name,s,concat(table.sortedkeys(ss), " ")) -- todo: padd 4
                     end
                 end
             end
@@ -468,13 +486,115 @@ function scripts.fonts.list()
 
 end
 
+-- For Mikael S:
+
+function scripts.fonts.coverage()
+
+    local coverage = getargument("coverage")
+    local reload   = getargument("reload")
+    local pattern  = getargument("pattern")
+
+    if type(coverage) ~= "string" or coverage == "" then
+        return
+    end
+
+    coverage = string.gsub(coverage,"0x([0-9A-Fa-f]+)",function(s)
+        return utf.char(tonumber(s,16))
+    end)
+
+    coverage = string.gsub(coverage,"U%+([0-9A-F]+)",function(s)
+        return utf.char(tonumber(s,16))
+    end)
+
+    coverage = string.gsub(coverage,"%s+","")
+
+    local chars = table.unique(utf.split(coverage))
+    local bytes = { }
+
+    for i=1,#chars do
+        bytes[#bytes+1] = utf.byte(chars[i])
+    end
+
+    local f_c = string.formatters["%C"]
+
+    local function okay(data)
+        if data then
+            local descriptions = data.descriptions
+            if descriptions then
+                local f = { }
+                local d = false
+                for i=1,#bytes do
+                    local b = bytes[i]
+                    if descriptions[b] then
+                        f[i] = f_c(b)
+                        d = true
+                    else
+                        f[i] = "[" .. f_c(b) .. "]"
+                    end
+                end
+                return d and f or false
+            end
+        end
+    end
+
+    reloadbase(reload)
+
+    local files = fonts.names.list(pattern and string.topattern(pattern,true) or "",reload,true)
+    local found = { }
+    local done  = { }
+
+--     inspect(files)
+
+    if files then
+        for id, specification in next, files do
+            if specification.format == "otf" or specification.format == "ttf" then
+                if not done[specification.filename] then
+                    local fullname = resolvers.findfile(specification.filename) or ""
+                    if fullname ~= "" then
+                        local data = fonts.handlers.otf.load(fullname)
+                        local list = okay(data)
+                        if list then
+                            found[id] = list
+                        end
+                    end
+                    done[specification.filename] = true
+                end
+            end
+    --     collectgarbage("collect")
+        end
+    end
+
+    if next(found) then
+        report()
+        for id, found in table.sortedhash(found) do
+            local specification = files[id]
+            indeed("filename  : %s",specification.filename)
+            indeed("fontname  : %s",specification.fontname)
+            indeed("coverage  : % t",found)
+            report()
+        end
+    end
+end
+
 function scripts.fonts.unpack()
     local name = removesuffix(basename(givenfiles[1] or ""))
     if name and name ~= "" then
-        local cacheid   = getargument("cache") or "otl"
-        local cache     = containers.define("fonts", cacheid, otlversion, true) -- cache is temp
-        local cleanname = containers.cleanname(name)
-        local data = containers.read(cache,cleanname)
+        local cacheid   = false
+        local cache     = false
+        local cleanname = false
+        local data      = false
+        local list = { getargument("cache") or false, "otl", "one" }
+        for i=1,#list do
+            cacheid   = list[i]
+            if cacheid then
+                cache     = containers.define("fonts", cacheid, versions[cacheid], true) -- cache is temp
+                cleanname = containers.cleanname(name)
+                data      = containers.read(cache,cleanname)
+                if data then
+                    break
+                end
+            end
+        end
         if data then
             local savename = addsuffix(cleanname .. "-unpacked","tma")
             report("fontsave, saving data in %s",savename)
@@ -534,6 +654,8 @@ elseif getargument("reload") then
     scripts.fonts.reload()
 elseif getargument("convert") then
     scripts.fonts.convert()
+elseif getargument("coverage") then
+    scripts.fonts.coverage()
 elseif getargument("unpack") then
     scripts.fonts.unpack()
 elseif getargument("statistics") then
