@@ -163,8 +163,7 @@ local function textopener(tag,filename,filehandle,coding)
             if trace_locating then
                 report_tex("%a closer: %a closed",tag,filename)
             end
-            handler = nil
-            lines   = nil
+            lines = nil
         end,
         reader      = function(self)
             self = self or handler
@@ -175,9 +174,10 @@ local function textopener(tag,filename,filehandle,coding)
                 currentline = currentline + 1
              -- self.currentline = currentline
                 local content = lines[currentline]
--- lines[currentline] = nil
+             -- lines[currentline] = false
                 if content == "" then
-                    return ""
+                 -- return ""
+                    return content
              -- elseif content == ctrl_d or ctrl_z then
              --     return nil -- we need this as \endinput does not work in prints
                 elseif content then
