@@ -41,18 +41,7 @@ local prefixes = {
 
 -- the order might change and how about cld
 
-local suffixes = CONTEXTLMTXMODE > 0 and
-{
-    "mklx", -- preprocessed mkiv lmtx files
-    "mkxl", -- mkiv lmtx files
-    "mkvi", -- preprocessed mkiv files
-    "mkiv", -- mkiv files
-    "tex",  -- normally source code files
-    "cld",  -- context lua documents (often stand alone)
-    "lua",  -- lua files
-}
-    or
-{
+local suffixes = {
     "mkvi",
     "mkiv",
     "tex",
@@ -195,7 +184,7 @@ end)
 logs.registerfinalactions(function()
     logs.startfilelogging(report,"used modules")
     for k, v in sortedhash(modstatus) do
-        report(v == 0 and "missing: %s" or "loaded : %s",basename(k))
+        report(v == 0 and "  missing: %s" or "  loaded : %s",basename(k))
     end
     logs.stopfilelogging()
     if missing and logs.loggingerrors() then
