@@ -29,7 +29,7 @@ if not modules then modules = { } end modules ['font-cff'] = {
 
 local next, type, tonumber, rawget = next, type, tonumber, rawget
 local byte, char, gmatch, sub = string.byte, string.char, string.gmatch, string.sub
-local concat, remove, unpack = table.concat, table.remove, table.unpack
+local concat, insert, remove, unpack = table.concat, table.insert, table.remove, table.unpack
 local floor, abs, round, ceil, min, max = math.floor, math.abs, math.round, math.ceil, math.min, math.max
 local P, C, R, S, C, Cs, Ct = lpeg.P, lpeg.C, lpeg.R, lpeg.S, lpeg.C, lpeg.Cs, lpeg.Ct
 local lpegmatch = lpeg.match
@@ -348,11 +348,11 @@ do
             top = 0
         end
       + P("\10") / function()
-            result.strhw = stack[top]
+            result.stdhw = stack[top]
             top = 0
         end
       + P("\11") / function()
-            result.strvw = stack[top]
+            result.stdvw = stack[top]
             top = 0
         end
       + P("\13") / function()
@@ -453,7 +453,7 @@ do
             top = 0
         end
       + P("\10") / function()
-            result.bluesnap = stack[top]
+            result.blueshift = stack[top]
             top = 0
         end
       + P("\11") / function()
@@ -528,7 +528,7 @@ do
     -- the second variant is much faster. Not that it matters much as we don't see
     -- such numbers often.
 
-    local remap = {
+    local remap_1 = {
         ["\x00"] = "00",  ["\x01"] = "01",  ["\x02"] = "02",  ["\x03"] = "03",  ["\x04"] = "04",  ["\x05"] = "05",  ["\x06"] = "06",  ["\x07"] = "07",  ["\x08"] = "08",  ["\x09"] = "09",  ["\x0A"] = "0.",  ["\x0B"] = "0E",  ["\x0C"] = "0E-",  ["\x0D"] = "0",  ["\x0E"] = "0-",  ["\x0F"] = "0",
         ["\x10"] = "10",  ["\x11"] = "11",  ["\x12"] = "12",  ["\x13"] = "13",  ["\x14"] = "14",  ["\x15"] = "15",  ["\x16"] = "16",  ["\x17"] = "17",  ["\x18"] = "18",  ["\x19"] = "19",  ["\x1A"] = "1.",  ["\x1B"] = "1E",  ["\x1C"] = "1E-",  ["\x1D"] = "1",  ["\x1E"] = "1-",  ["\x1F"] = "1",
         ["\x20"] = "20",  ["\x21"] = "21",  ["\x22"] = "22",  ["\x23"] = "23",  ["\x24"] = "24",  ["\x25"] = "25",  ["\x26"] = "26",  ["\x27"] = "27",  ["\x28"] = "28",  ["\x29"] = "29",  ["\x2A"] = "2.",  ["\x2B"] = "2E",  ["\x2C"] = "2E-",  ["\x2D"] = "2",  ["\x2E"] = "2-",  ["\x2F"] = "2",
@@ -544,11 +544,18 @@ do
         ["\xC0"] = "E-0", ["\xC1"] = "E-1", ["\xC2"] = "E-2", ["\xC3"] = "E-3", ["\xC4"] = "E-4", ["\xC5"] = "E-5", ["\xC6"] = "E-6", ["\xC7"] = "E-7", ["\xC8"] = "E-8", ["\xC9"] = "E-9", ["\xCA"] = "E-.", ["\xCB"] = "E-E", ["\xCC"] = "E-E-", ["\xCD"] = "E-", ["\xCE"] = "E--", ["\xCF"] = "E-",
         ["\xD0"] = "-0",  ["\xD1"] = "-1",  ["\xD2"] = "-2",  ["\xD3"] = "-3",  ["\xD4"] = "-4",  ["\xD5"] = "-5",  ["\xD6"] = "-6",  ["\xD7"] = "-7",  ["\xD8"] = "-8",  ["\xD9"] = "-9",  ["\xDA"] = "-.",  ["\xDB"] = "-E",  ["\xDC"] = "-E-",  ["\xDD"] = "-",  ["\xDE"] = "--",  ["\xDF"] = "-",
     }
+    local remap_2 = {
+        ["\x0F"] = "0", ["\x1F"] = "1", ["\x2F"] = "2", ["\x3F"] = "3", ["\x4F"] = "4",
+        ["\x5F"] = "5", ["\x6F"] = "6", ["\x7F"] = "7", ["\x8F"] = "8", ["\x9F"] = "9",
+    }
 
-    local p_last = S("\x0F\x1F\x2F\x3F\x4F\x5F\x6F\x7F\x8F\x9F\xAF\xBF")
-                 + R("\xF0\xFF")
+    local p_last_1 = S("\x0F\x1F\x2F\x3F\x4F\x5F\x6F\x7F\x8F\x9F\xAF\xBF")
+    local p_last_2 = R("\xF0\xFF")
 
-    local p_nibbles = P("\30") * Cs(((1-p_last)/remap)^0 * (P(1)/remap)) / function(n)
+    -- tricky, we don't want to append last
+
+ -- local p_nibbles = P("\30") * Cs(((1-p_last)/remap)^0 * (P(1)/remap)) / function(n)
+    local p_nibbles = P("\30") * Cs(((1-(p_last_1+p_last_2))/remap_1)^0 * (p_last_1/remap_2 + p_last_2/"")) / function(n)
         -- 0-9=digit a=. b=E c=E- d=reserved e=- f=finish
         top = top + 1
         stack[top] = tonumber(n) or 0
@@ -572,6 +579,11 @@ do
         stack[top] = -(byte(b0)-251)*256 - byte(b1) - 108
     end
 
+ -- local p_float = P("\255") * C(1) * C(1) * C(1) * C(1) / function(b0,b1,b2,b3)
+ --     top = top + 1
+ --     stack[top] = 0
+ -- end
+
     local p_short = P("\28") * C(1) * C(1) / function(b1,b2)
         -- -32768 .. +32767 : b1<<8 | b2
         top = top + 1
@@ -607,6 +619,7 @@ do
       + p_nibbles
       + p_single
       + p_double
+   -- + p_float
       + p_unsupported
     )^1
 
@@ -701,6 +714,7 @@ do
     local y            = 0
     local width        = false
     local lsb          = 0
+    local result       = { }
     local r            = 0
     local stems        = 0
     local globalbias   = 0
@@ -728,8 +742,13 @@ do
     local seacs        = { }
     local procidx      = nil
 
-    local function showstate(where)
-        report("%w%-10s : [%s] n=%i",depth*2,where,concat(stack," ",1,top),top)
+    local function showstate(where,i,n)
+        if i then
+            local j = i + n - 1
+            report("%w%-10s : [%s] step",depth*2+2,where,concat(stack," ",i,j <= top and j or top))
+        else
+            report("%w%-10s : [%s] n=%i",depth*2,where,concat(stack," ",1,top),top)
+        end
     end
 
     local function showvalue(where,value,showstack)
@@ -745,6 +764,13 @@ do
     -- some back. I inlined some of then and a bit speed can be gained by more
     -- inlining but not that much.
 
+    -- Maybe have several action tables:
+    --
+    -- keep curve / checked
+    -- keep curve / not checked
+    -- checked
+    -- not checked
+
     local function xymoveto()
         if keepcurve then
             r = r + 1
@@ -859,9 +885,9 @@ do
         end
     end
 
-    local function xycurveto(x1,y1,x2,y2,x3,y3) -- called local so no blend here
+    local function xycurveto(x1,y1,x2,y2,x3,y3,i,n) -- called local so no blend here
         if trace_charstrings then
-            showstate("curveto")
+            showstate("curveto",i,n)
         end
         if keepcurve then
             r = r + 1
@@ -1005,6 +1031,16 @@ do
         if trace_charstrings then
             showstate("rrcurveto")
         end
+if top == 6 then
+    local ax = x  + stack[1] -- dxa
+    local ay = y  + stack[2] -- dya
+    local bx = ax + stack[3] -- dxb
+    local by = ay + stack[4] -- dyb
+    x = bx + stack[5]        -- dxc
+    y = by + stack[6]        -- dyc
+    xycurveto(ax,ay,bx,by,x,y,1,6)
+else
+-- print("rr",top==6,top)
         for i=1,top,6 do
             local ax = x  + stack[i]   -- dxa
             local ay = y  + stack[i+1] -- dya
@@ -1012,8 +1048,9 @@ do
             local by = ay + stack[i+3] -- dyb
             x = bx + stack[i+4]        -- dxc
             y = by + stack[i+5]        -- dyc
-            xycurveto(ax,ay,bx,by,x,y)
+            xycurveto(ax,ay,bx,by,x,y,i,6)
         end
+end
         top = 0
     end
 
@@ -1026,6 +1063,15 @@ do
             y = y + stack[1]           -- dy1
             s = 2
         end
+if top == 4 then
+            local ax = x + stack[1]  -- dxa
+            local ay = y
+            local bx = ax + stack[2] -- dxb
+            local by = ay + stack[3] -- dyb
+            x = bx + stack[4]        -- dxc
+            y = by
+            xycurveto(ax,ay,bx,by,x,y,1,4)
+else
         for i=s,top,4 do
             local ax = x + stack[i]    -- dxa
             local ay = y
@@ -1033,8 +1079,9 @@ do
             local by = ay + stack[i+2] -- dyb
             x = bx + stack[i+3]        -- dxc
             y = by
-            xycurveto(ax,ay,bx,by,x,y)
+            xycurveto(ax,ay,bx,by,x,y,i,4)
         end
+end
         top = 0
     end
 
@@ -1048,6 +1095,16 @@ do
             d = stack[1]               -- dx1
             s = 2
         end
+if top == 4 then
+    local ax = x + d
+    local ay = y + stack[1]  -- dya
+    local bx = ax + stack[2] -- dxb
+    local by = ay + stack[3] -- dyb
+    x = bx
+    y = by + stack[4]        -- dyc
+    xycurveto(ax,ay,bx,by,x,y,1,4)
+    d = 0
+else
         for i=s,top,4 do
             local ax = x + d
             local ay = y + stack[i]    -- dya
@@ -1055,9 +1112,10 @@ do
             local by = ay + stack[i+2] -- dyb
             x = bx
             y = by + stack[i+3]        -- dyc
-            xycurveto(ax,ay,bx,by,x,y)
+            xycurveto(ax,ay,bx,by,x,y,i,4)
             d = 0
         end
+end
         top = 0
     end
 
@@ -1066,6 +1124,33 @@ do
         if last then
             top = top - 1
         end
+if top == 4 then
+        local ax, ay, bx, by
+        if swap then
+            ax = x  + stack[1]
+            ay = y
+            bx = ax + stack[2]
+            by = ay + stack[3]
+            y  = by + stack[4]
+            if last then
+                x = bx + last
+            else
+                x = bx
+            end
+        else
+            ax = x
+            ay = y  + stack[1]
+            bx = ax + stack[2]
+            by = ay + stack[3]
+            x  = bx + stack[4]
+            if last then
+                y = by + last
+            else
+                y = by
+            end
+        end
+        xycurveto(ax,ay,bx,by,x,y,1 ,4)
+else
         for i=1,top,4 do
             local ax, ay, bx, by
             if swap then
@@ -1093,8 +1178,9 @@ do
                 end
                 swap = true
             end
-            xycurveto(ax,ay,bx,by,x,y)
+            xycurveto(ax,ay,bx,by,x,y,i,4)
         end
+end
         top = 0
     end
 
@@ -1123,7 +1209,7 @@ do
             local by = ay + stack[i+3] -- dyb
             x = bx + stack[i+4] -- dxc
             y = by + stack[i+5] -- dyc
-            xycurveto(ax,ay,bx,by,x,y)
+            xycurveto(ax,ay,bx,by,x,y,i,6)
         end
         x = x + stack[top-1] -- dxc
         y = y + stack[top]   -- dyc
@@ -1628,7 +1714,8 @@ do
 
     -- todo: round in blend
 
-    local encode = { }
+    local encode  = { }
+    local typeone = false
 
     -- this eventually can become a helper
 
@@ -1645,15 +1732,13 @@ do
         end
         for i=108,1131 do
             local v = 0xF700 + i - 108
---             t[i] = char(band(rshift(v,8),0xFF),band(v,0xFF))
             t[i] = char(extract(v,8,8),extract(v,0,8))
         end
         for i=1132,2048 do
             t[i] = char(28,band(rshift(i,8),0xFF),band(i,0xFF))
         end
-        -- we could inline some ...
         setmetatableindex(encode,function(t,k)
-            -- 16.16-bit signed fixed value
+            -- as we're cff2 we write 16.16-bit signed fixed value
             local r = round(k)
             local v = rawget(t,r)
             if v then
@@ -1758,42 +1843,6 @@ do
 
     -- precompiling and reuse is much slower than redoing the calls
 
- -- local function decode(str)
- --     local a, b, c, d, e = byte(str,1,5)
- --     if a == 28 then
- --         if c then
- --             local n = 0x100 * b + c
- --             if n >= 0x8000 then
- --                 return n - 0x10000
- --             else
- --                 return n
- --             end
- --         end
- --     elseif a < 32 then
- --         return false
- --     elseif a <= 246 then
- --         return  a - 139
- --     elseif a <= 250 then
- --         if b then
- --             return  a*256 - 63124 + b
- --         end
- --     elseif a <= 254 then
- --         if b then
- --             return -a*256 + 64148 - b
- --         end
- --     else
- --         if e then
- --             local n = 0x100 * b + c
- --             if n >= 0x8000 then
- --                 return n - 0x10000 + (0x100 * d + e)/0xFFFF
- --             else
- --                 return n           + (0x100 * d + e)/0xFFFF
- --             end
- --         end
- --     end
- --     return false
- -- end
-
     process = function(tab)
         local i = 1
         local n = #tab
@@ -1817,14 +1866,20 @@ do
                  -- stack[top] = -t*256 + 251*256 - tab[i+1] - 108
                     stack[top] = -t*256 + 64148 - tab[i+1]
                     i = i + 2
-                else
-                    -- a 16.16 float
-                    local n = 0x100 * tab[i+1] + tab[i+2]
-                    if n >= 0x8000 then
-                        stack[top] = n - 0x10000 + (0x100 * tab[i+3] + tab[i+4])/0xFFFF
-                    else
-                        stack[top] = n           + (0x100 * tab[i+3] + tab[i+4])/0xFFFF
+                elseif typeone then
+                    local n = 0x1000000 * tab[i+1] + 0x10000 * tab[i+2] + 0x100 * tab[i+3] + tab[i+4]
+                    if n >= 0x8000000 then
+                        n = n - 0xFFFFFFFF - 1
                     end
+                    stack[top] = n
+                    i = i + 5
+                else
+                    local n1 = 0x100 * tab[i+1] + tab[i+2]
+                    local n2 = 0x100 * tab[i+3] + tab[i+4]
+                    if n1 >= 0x8000 then
+                        n1 = n1 - 0x10000
+                    end
+                    stack[top] = n1 + n2/0xFFFF
                     i = i + 5
                 end
             elseif t == 28 then
@@ -1917,7 +1972,8 @@ do
                 -- cff 1: (when cff2 strip them)
                 elseif t == 1 or t == 3 or t == 18 or operation == 23 then
                     p_getstem() -- at the start
-                    if true then
+                    if version == "cff" then
+--                     if true then
                         if top > 0 then
                             for i=1,top do
                                 r = r + 1 ; result[r] = encode[stack[i]]
@@ -1932,6 +1988,7 @@ do
                 -- cff 1: (when cff2 strip them)
                 elseif t == 19 or t == 20 then
                     local s = p_getmask() or 0 -- after the stems
+--                     if version == "cff" then
                     if true then
                         if top > 0 then
                             for i=1,top do
@@ -1955,7 +2012,8 @@ do
                     i = i + 1
                 elseif t == 13 then
                     hsbw()
-                    if version == "cff" then
+--                     if version == "cff" then
+                    if true then
                         -- we do a moveto over lsb
                         r = r + 1 ; result[r] = encode[lsb]
                         r = r + 1 ; result[r] = chars[22]
@@ -1968,8 +2026,40 @@ do
                         showstate(reverse[t] or "<action>")
                     end
                     if top > 0 then
-                        for i=1,top do
-                            r = r + 1 ; result[r] = encode[stack[i]]
+                     -- if t == 8 and top > 42 then
+                        if t == 8 and top > 48 then
+                            -- let's assume this only happens for rrcurveto .. the other ones would need some more
+                            -- complex handling (cff2 stuff)
+                            --
+                            -- dx1 dy1 (dx1+dx2) (dy1+dy2) (dx1+dx2+dx3) (dy1+dy2+dy3) rcurveto.
+                            local n = 0
+                            for i=1,top do
+                             -- if n == 42 then
+                                if n == 48 then
+--                                     local zero = encode[0]
+--                                     local res3 = result[r-3]
+--                                     local res2 = result[r-2]
+--                                     local res1 = result[r-1]
+--                                     local res0 = result[r]
+--                                     result[r-3] = zero
+--                                     result[r-2] = zero
+                                    r = r + 1 ; result[r] = chars[t]
+--                                     r = r + 1 ; result[r] = zero
+--                                     r = r + 1 ; result[r] = zero
+--                                     r = r + 1 ; result[r] = res3
+--                                     r = r + 1 ; result[r] = res2
+--                                     r = r + 1 ; result[r] = res1
+--                                     r = r + 1 ; result[r] = res0
+                                    n = 1
+                                else
+                                    n = n + 1
+                                end
+                                r = r + 1 ; result[r] = encode[stack[i]]
+                            end
+                        else
+                            for i=1,top do
+                                r = r + 1 ; result[r] = encode[stack[i]]
+                            end
                         end
                         top = 0
                     end
@@ -2046,7 +2136,7 @@ do
         end
     end
 
-    local function processshape(tab,index,hack)
+    local function processshape(glyphs,tab,index,hack)
 
         if not tab then
             glyphs[index] = {
@@ -2085,7 +2175,6 @@ do
         end
 
         process(tab)
-
         if hack then
             return x, y
         end
@@ -2108,13 +2197,15 @@ do
             r = r + 1
             result[r] = c_endchar
             local stream = concat(result)
+result = nil
          -- if trace_charstrings then
          --     report("vdata: %s",stream)
          -- end
             if glyph then
-                glyph.stream  = stream
+                glyph.stream = stream
+                glyph.width  = width
             else
-                glyphs[index] = { stream = stream }
+                glyphs[index] = { stream = stream, width = width }
             end
         elseif glyph then
             glyph.segments    = keepcurve ~= false and result or nil
@@ -2134,6 +2225,7 @@ do
                 name        = charset and charset[index] or nil,
              -- sidebearing = 0,
             }
+result = nil
         else
             glyphs[index] = {
                 boundingbox = boundingbox,
@@ -2141,7 +2233,6 @@ do
                 name        = charset and charset[index] or nil,
             }
         end
-
         if trace_charstrings then
             report("width      : %s",tostring(width))
             report("boundingbox: % t",boundingbox)
@@ -2192,19 +2283,21 @@ do
         return privatedata.nominalwidthx or 0, privatedata.defaultwidthx or 0
     end
 
-    parsecharstrings = function(fontdata,data,glphs,doshapes,tversion,streams,nobias)
+    parsecharstrings = function(fontdata,data,glphs,doshapes,tversion,streams,nobias,istypeone)
 
         local dictionary  = data.dictionaries[1]
         local charstrings = dictionary.charstrings
 
         keepcurve = doshapes
         version   = tversion
+        typeone   = istypeone or false
         strings   = data.strings
         globals   = data.routines or { }
         locals    = dictionary.subroutines or { }
         charset   = dictionary.charset
         vsindex   = dictionary.vsindex or 0
-        glyphs    = glphs or { }
+
+        local glyphs = glphs or { }
 
         globalbias,   localbias    = setbias(globals,locals,nobias)
         nominalwidth, defaultwidth = setwidths(dictionary.private)
@@ -2212,7 +2305,7 @@ do
         if charstrings then
             startparsing(fontdata,data,streams)
             for index=1,#charstrings do
-                processshape(charstrings[index],index-1)
+                processshape(glyphs,charstrings[index],index-1)
             end
             if justpass and next(seacs) then
                 -- old type 1 stuff ... seacs
@@ -2228,7 +2321,7 @@ do
                             -- this is a real ugly hack but we seldom enter this branch (e.g. old lbr)
                             local jp = justpass
                             justpass = false
-                            local x, y = processshape(charstrings[bindex+1],bindex,true)
+                            local x, y = processshape(glyphs,charstrings[bindex+1],bindex,true)
                             justpass = jp
                             --
                             local base   = bglyph.stream
@@ -2259,7 +2352,8 @@ do
         locals    = dictionary.subroutines or { }
         charset   = false
         vsindex   = dictionary.vsindex or 0
-        glyphs    = glphs or { }
+
+        local glyphs = glphs or { }
 
         justpass = streams == true
         seacs    = { }
@@ -2267,9 +2361,9 @@ do
         globalbias,   localbias    = setbias(globals,locals,nobias)
         nominalwidth, defaultwidth = setwidths(dictionary.private)
 
-        processshape(tab,index-1)
+        processshape(glyphs,tab,index-1)
 
-     -- return glyphs[index]
+        return glyphs[index]
     end
 
 end
@@ -2434,7 +2528,7 @@ local function readnoselect(f,fontdata,data,glyphs,doshapes,version,streams)
     parseprivates(data,data.dictionaries)
     readlocals(f,data,dictionary,version)
     startparsing(fontdata,data,streams)
-    parsecharstrings(fontdata,data,glyphs,doshapes,version,streams)
+    parsecharstrings(fontdata,data,glyphs,doshapes,version,streams,false)
     stopparsing(fontdata,data)
 end
 
@@ -2483,7 +2577,7 @@ local function readfdselect(f,fontdata,data,glyphs,doshapes,version,streams)
             end
         end
     else
-        -- unsupported format
+        report("unsupported fd index format %i",format)
     end
     -- hm, always
     if maxindex >= 0 then
@@ -2491,21 +2585,30 @@ local function readfdselect(f,fontdata,data,glyphs,doshapes,version,streams)
         if cidarray then
             setposition(f,header.offset+cidarray)
             local dictionaries = readlengths(f,version == "cff2")
-            for i=1,#dictionaries do
-                dictionaries[i] = readstring(f,dictionaries[i])
+            if #dictionaries > 0 then
+                for i=1,#dictionaries do
+                    dictionaries[i] = readstring(f,dictionaries[i])
+                end
+                parsedictionaries(data,dictionaries)
+                cid.dictionaries = dictionaries
+                readcidprivates(f,data)
+                for i=1,#dictionaries do
+                    readlocals(f,data,dictionaries[i],version)
+                end
+                startparsing(fontdata,data,streams)
+                for i=1,#charstrings do
+                    local dictionary = dictionaries[fdindex[i]+1]
+                    if dictionary then
+                        parsecharstring(fontdata,data,dictionary,charstrings[i],glyphs,i,doshapes,version,streams)
+                    else
+                     -- report("no dictionary for %a : %a => %a",version,i,fdindex[i]+1)
+                    end
+                 -- charstrings[i] = false
+                end
+                stopparsing(fontdata,data)
+            else
+                report("no cid dictionaries")
             end
-            parsedictionaries(data,dictionaries)
-            cid.dictionaries = dictionaries
-            readcidprivates(f,data)
-            for i=1,#dictionaries do
-                readlocals(f,data,dictionaries[i],version)
-            end
-            startparsing(fontdata,data,streams)
-            for i=1,#charstrings do
-                parsecharstring(fontdata,data,dictionaries[fdindex[i]+1],charstrings[i],glyphs,i,doshapes,version,streams)
---                 charstrings[i] = nil
-            end
-            stopparsing(fontdata,data)
         else
             report("no cid array")
         end
@@ -2593,6 +2696,8 @@ function readers.cff(f,fontdata,specification)
                 cffinfo.bluefuzz         = data.bluefuzz
                 cffinfo.stdhw            = data.stdhw
                 cffinfo.stdvw            = data.stdvw
+                cffinfo.stemsnaph        = data.stemsnaph
+                cffinfo.stemsnapv        = data.stemsnapv
             end
         end
         cleanup(data,dictionaries)
@@ -2643,38 +2748,38 @@ end
 
 -- temporary helper needed for checking backend patches
 
-function readers.cffcheck(filename)
-    local f = io.open(filename,"rb")
-    if f then
-        local fontdata = {
-            glyphs = { },
-        }
-        local header = readheader(f)
-        if header.major ~= 1 then
-            report("only version %s is supported for table %a",1,"cff")
-            return
-        end
-        local names        = readfontnames(f)
-        local dictionaries = readtopdictionaries(f)
-        local strings      = readstrings(f)
-        local glyphs       = { }
-        local data = {
-            header       = header,
-            names        = names,
-            dictionaries = dictionaries,
-            strings      = strings,
-            glyphs       = glyphs,
-            nofglyphs    = 0,
-        }
-        --
-        parsedictionaries(data,dictionaries,"cff")
-        --
-        local cid = data.dictionaries[1].cid
-        if cid and cid.fdselect then
-            readfdselect(f,fontdata,data,glyphs,false)
-        else
-            readnoselect(f,fontdata,data,glyphs,false)
-        end
-        return data
-    end
-end
+-- function readers.cffcheck(filename)
+--     local f = io.open(filename,"rb")
+--     if f then
+--         local fontdata = {
+--             glyphs = { },
+--         }
+--         local header = readheader(f)
+--         if header.major ~= 1 then
+--             report("only version %s is supported for table %a",1,"cff")
+--             return
+--         end
+--         local names        = readfontnames(f)
+--         local dictionaries = readtopdictionaries(f)
+--         local strings      = readstrings(f)
+--         local glyphs       = { }
+--         local data = {
+--             header       = header,
+--             names        = names,
+--             dictionaries = dictionaries,
+--             strings      = strings,
+--             glyphs       = glyphs,
+--             nofglyphs    = 0,
+--         }
+--         --
+--         parsedictionaries(data,dictionaries,"cff")
+--         --
+--         local cid = data.dictionaries[1].cid
+--         if cid and cid.fdselect then
+--             readfdselect(f,fontdata,data,glyphs,false)
+--         else
+--             readnoselect(f,fontdata,data,glyphs,false)
+--         end
+--         return data
+--     end
+-- end
