@@ -10,10 +10,6 @@ local cos, tan, rad, format = math.cos, math.tan, math.rad, string.format
 local utfbyte, utfchar = utf.byte, utf.char
 local next = next
 
---[[ldx--
-<p>This is very experimental code!</p>
---ldx]]--
-
 local trace_visualize    = false  trackers.register("fonts.composing.visualize", function(v) trace_visualize = v end)
 local trace_define       = false  trackers.register("fonts.composing.define",    function(v) trace_define    = v end)
 
@@ -189,6 +185,7 @@ local function composecharacters(tfmdata)
                                                     chr_t,
                                                 }
                                             end
+t.depth = a_ury
                                         elseif c_ury > a_lly then -- messy test
                                             local dy
                                             if compose then
@@ -219,6 +216,7 @@ local function composecharacters(tfmdata)
                                             else
                                                 dy = - deltaxheight + extraxheight
                                             end
+t.height = a_ury-dy
                                             local right = rightcommand[dx+dd]
                                             local down  = downcommand[dy]
                                             if trace_visualize then
@@ -245,6 +243,7 @@ local function composecharacters(tfmdata)
                                                     chr_t,
                                                 }
                                             end
+t.height = a_ury
                                         end
                                     end
                                 else
