@@ -1,23 +1,22 @@
 if not modules then modules = { } end modules ['font-one'] = {
     version   = 1.001,
+    optimize  = true,
     comment   = "companion to font-ini.mkiv",
     author    = "Hans Hagen, PRAGMA-ADE, Hasselt NL",
     copyright = "PRAGMA ADE / ConTeXt Development Team",
     license   = "see context related readme files"
 }
 
---[[ldx--
-<p>Some code may look a bit obscure but this has to do with the fact that we also use
-this code for testing and much code evolved in the transition from <l n='tfm'/> to
-<l n='afm'/> to <l n='otf'/>.</p>
-
-<p>The following code still has traces of intermediate font support where we handles
-font encodings. Eventually font encoding went away but we kept some code around in
-other modules.</p>
-
-<p>This version implements a node mode approach so that users can also more easily
-add features.</p>
---ldx]]--
+-- Some code may look a bit obscure but this has to do with the fact that we also
+-- use this code for testing and much code evolved in the transition from TFM to AFM
+-- to OTF.
+--
+-- The following code still has traces of intermediate font support where we handles
+-- font encodings. Eventually font encoding went away but we kept some code around
+-- in other modules.
+--
+-- This version implements a node mode approach so that users can also more easily
+-- add features.
 
 local fonts, logs, trackers, containers, resolvers = fonts, logs, trackers, containers, resolvers
 
@@ -59,7 +58,7 @@ local registerafmfeature  = afmfeatures.register
 local afmenhancers        = constructors.enhancers.afm
 local registerafmenhancer = afmenhancers.register
 
-afm.version               = 1.513 -- incrementing this number one up will force a re-cache
+afm.version               = 1.541 -- incrementing this number one up will force a re-cache
 afm.cache                 = containers.define("fonts", "one", afm.version, true)
 afm.autoprefixed          = true -- this will become false some day (catches texnansi-blabla.*)
 
@@ -70,15 +69,13 @@ local overloads           = fonts.mappings.overloads
 
 local applyruntimefixes   = fonts.treatments and fonts.treatments.applyfixes
 
---[[ldx--
-<p>We cache files. Caching is taken care of in the loader. We cheat a bit by adding
-ligatures and kern information to the afm derived data. That way we can set them faster
-when defining a font.</p>
-
-<p>We still keep the loading two phased: first we load the data in a traditional
-fashion and later we transform it to sequences. Then we apply some methods also
-used in opentype fonts (like <t>tlig</t>).</p>
---ldx]]--
+-- We cache files. Caching is taken care of in the loader. We cheat a bit by adding
+-- ligatures and kern information to the afm derived data. That way we can set them
+-- faster when defining a font.
+--
+-- We still keep the loading two phased: first we load the data in a traditional
+-- fashion and later we transform it to sequences. Then we apply some methods also
+-- used in opentype fonts (like tlig).
 
 function afm.load(filename)
     filename = resolvers.findfile(filename,'afm') or ""
@@ -86,14 +83,16 @@ function afm.load(filename)
         local name = file.removesuffix(file.basename(filename))
         local data = containers.read(afm.cache,name)
         local attr = lfs.attributes(filename)
-        local size, time = attr.size or 0, attr.modification or 0
+        local size = attr and attr.size or 0
+        local time = attr and attr.modification or 0
         --
         local pfbfile = file.replacesuffix(name,"pfb")
         local pfbname = resolvers.findfile(pfbfile,"pfb") or ""
         if pfbname == "" then
             pfbname = resolvers.findfile(file.basename(pfbfile),"pfb") or ""
         end
-        local pfbsize, pfbtime = 0, 0
+        local pfbsize = 0
+        local pfbtime = 0
         if pfbname ~= "" then
             local attr = lfs.attributes(pfbname)
             pfbsize = attr.size or 0
@@ -106,6 +105,7 @@ function afm.load(filename)
                 afmenhancers.apply(data,filename)
              -- otfreaders.addunicodetable(data) -- only when not done yet
                 fonts.mappings.addtounicode(data,filename)
+                otfreaders.stripredundant(data)
              -- otfreaders.extend(data)
                 otfreaders.pack(data)
                 data.size = size
@@ -308,10 +308,8 @@ local function enhance_fix_names(data)
     end
 end
 
---[[ldx--
-<p>These helpers extend the basic table with extra ligatures, texligatures
-and extra kerns. This saves quite some lookups later.</p>
---ldx]]--
+-- These helpers extend the basic table with extra ligatures, texligatures and extra
+-- kerns. This saves quite some lookups later.
 
 local addthem = function(rawdata,ligatures)
     if ligatures then
@@ -323,7 +321,8 @@ local addthem = function(rawdata,ligatures)
             local one = descriptions[unicodes[ligname]]
             if one then
                 for _, pair in next, ligdata do
-                    local two, three = unicodes[pair[1]], unicodes[pair[2]]
+                    local two   = unicodes[pair[1]]
+                    local three = unicodes[pair[2]]
                     if two and three then
                         local ol = one.ligatures
                         if ol then
@@ -344,17 +343,14 @@ local function enhance_add_ligatures(rawdata)
     addthem(rawdata,afm.helpdata.ligatures)
 end
 
---[[ldx--
-<p>We keep the extra kerns in separate kerning tables so that we can use
-them selectively.</p>
---ldx]]--
-
--- This is rather old code (from the beginning when we had only tfm). If
--- we unify the afm data (now we have names all over the place) then
--- we can use shcodes but there will be many more looping then. But we
--- could get rid of the tables in char-cmp then. Als, in the generic version
--- we don't use the character database. (Ok, we can have a context specific
--- variant).
+-- We keep the extra kerns in separate kerning tables so that we can use them
+-- selectively.
+--
+-- This is rather old code (from the beginning when we had only tfm). If we unify
+-- the afm data (now we have names all over the place) then we can use shcodes but
+-- there will be many more looping then. But we could get rid of the tables in
+-- char-cmp then. Als, in the generic version we don't use the character database.
+-- (Ok, we can have a context specific variant).
 
 local function enhance_add_extra_kerns(rawdata) -- using shcodes is not robust here
     local descriptions = rawdata.descriptions
@@ -391,7 +387,7 @@ local function enhance_add_extra_kerns(rawdata) -- using shcodes is not robust h
         if what then
             for complex, simple in next, what do
                 complex = unicodes[complex]
-                simple = unicodes[simple]
+                simple  = unicodes[simple]
                 if complex and simple then
                     local complexdescription = descriptions[complex]
                     if complexdescription then -- optional
@@ -435,16 +431,15 @@ local function enhance_add_extra_kerns(rawdata) -- using shcodes is not robust h
     do_it_copy(afm.helpdata.rightkerned)
 end
 
---[[ldx--
-<p>The copying routine looks messy (and is indeed a bit messy).</p>
---ldx]]--
+-- The copying routine looks messy (and is indeed a bit messy).
 
 local function adddimensions(data) -- we need to normalize afm to otf i.e. indexed table instead of name
     if data then
         for unicode, description in next, data.descriptions do
             local bb = description.boundingbox
             if bb then
-                local ht, dp = bb[4], -bb[2]
+                local ht =  bb[4]
+                local dp = -bb[2]
                 if ht == 0 or ht < 0 then
                     -- no need to set it and no negative heights, nil == 0
                 else
@@ -478,8 +473,9 @@ local function copytotfm(data)
         local filename   = constructors.checkedfilename(resources)
         local fontname   = metadata.fontname or metadata.fullname
         local fullname   = metadata.fullname or metadata.fontname
-        local endash     = 0x0020 -- space
+        local endash     = 0x2013
         local emdash     = 0x2014
+        local space      = 0x0020 -- space
         local spacer     = "space"
         local spaceunits = 500
         --
@@ -491,28 +487,33 @@ local function copytotfm(data)
         parameters.italicangle = italicangle
         parameters.charwidth   = charwidth
         parameters.charxheight = charxheight
-        -- same as otf
+        -- nearly the same as otf, catches
+        local d_endash = descriptions[endash]
+        local d_emdash = descriptions[emdash]
+        local d_space  = descriptions[space]
+        if not d_space or d_space == 0 then
+            d_space = d_endash
+        end
+        if d_space then
+            spaceunits, spacer = d_space.width or 0, "space"
+        end
         if properties.monospaced then
-            if descriptions[endash] then
-                spaceunits, spacer = descriptions[endash].width, "space"
-            end
-            if not spaceunits and descriptions[emdash] then
-                spaceunits, spacer = descriptions[emdash].width, "emdash"
-            end
-            if not spaceunits and charwidth then
-                spaceunits, spacer = charwidth, "charwidth"
+            if spaceunits == 0 and d_emdash then
+                spaceunits, spacer = d_emdash.width or 0, "emdash"
             end
         else
-            if descriptions[endash] then
-                spaceunits, spacer = descriptions[endash].width, "space"
-            end
-            if not spaceunits and charwidth then
-                spaceunits, spacer = charwidth, "charwidth"
+            if spaceunits == 0 and d_endash then
+                spaceunits, spacer = d_emdash.width or 0, "endash"
             end
         end
-        spaceunits = tonumber(spaceunits)
-        if spaceunits < 200 then
-            -- todo: warning
+        if spaceunits == 0 and charwidth then
+            spaceunits, spacer = charwidth or 0, "charwidth"
+        end
+        if spaceunits == 0 then
+            spaceunits = tonumber(spaceunits) or 500
+        end
+        if spaceunits == 0 then
+            spaceunits = 500
         end
         --
         parameters.slant         = 0
@@ -581,7 +582,6 @@ local function copytotfm(data)
         parameters.units      = 1000
         --
         properties.spacer        = spacer
-        properties.encodingbytes = 2
         properties.format        = fonts.formats[filename] or "type1"
         properties.filename      = filename
         properties.fontname      = fontname
@@ -589,6 +589,7 @@ local function copytotfm(data)
         properties.psname        = fullname
         properties.name          = filename or fullname or fontname
         properties.private       = properties.private or data.private or privateoffset
+        properties.encodingbytes = 2
         --
         if next(characters) then
             return {
@@ -604,11 +605,9 @@ local function copytotfm(data)
     return nil
 end
 
---[[ldx--
-<p>Originally we had features kind of hard coded for <l n='afm'/> files but since I
-expect to support more font formats, I decided to treat this fontformat like any
-other and handle features in a more configurable way.</p>
---ldx]]--
+-- Originally we had features kind of hard coded for AFM files but since I expect to
+-- support more font formats, I decided to treat this fontformat like any other and
+-- handle features in a more configurable way.
 
 function afm.setfeatures(tfmdata,features)
     local okay = constructors.initializefeatures("afm",tfmdata,features,trace_features,report_afm)
@@ -700,18 +699,16 @@ local function afmtotfm(specification)
     end
 end
 
---[[ldx--
-<p>As soon as we could intercept the <l n='tfm'/> reader, I implemented an
-<l n='afm'/> reader. Since traditional <l n='pdftex'/> could use <l n='opentype'/>
-fonts with <l n='afm'/> companions, the following method also could handle
-those cases, but now that we can handle <l n='opentype'/> directly we no longer
-need this features.</p>
---ldx]]--
+-- As soon as we could intercept the TFM reader, I implemented an AFM reader. Since
+-- traditional pdfTeX could use OpenType fonts with AFM companions, the following
+-- method also could handle those cases, but now that we can handle OpenType
+-- directly we no longer need this features.
 
 local function read_from_afm(specification)
     local tfmdata = afmtotfm(specification)
     if tfmdata then
         tfmdata.properties.name = specification.name
+        tfmdata.properties.id   = specification.id
         tfmdata = constructors.scale(tfmdata, specification)
         local allfeatures = tfmdata.shared.features or specification.features.normal
         constructors.applymanipulators("afm",tfmdata,allfeatures,trace_features,report_afm)
@@ -720,9 +717,7 @@ local function read_from_afm(specification)
     return tfmdata
 end
 
---[[ldx--
-<p>We have the usual two modes and related features initializers and processors.</p>
---ldx]]--
+-- We have the usual two modes and related features initializers and processors.
 
 registerafmfeature {
     name         = "mode",
@@ -756,7 +751,7 @@ local function check_afm(specification,fullname)
     if foundname == "" then
         foundname = fonts.names.getfilename(fullname,"afm") or ""
     end
-    if foundname == "" and afm.autoprefixed then
+    if fullname and foundname == "" and afm.autoprefixed then
         local encoding, shortname = match(fullname,"^(.-)%-(.*)$") -- context: encoding-name.*
         if encoding and shortname and fonts.encodings.known[encoding] then
             shortname = findbinfile(shortname,'afm') or "" -- just to be sure
