@@ -22,9 +22,10 @@ local otf                 = fonts.handlers.otf
 local registerotffeature  = otf.features.register
 local setmetatableindex   = table.setmetatableindex
 
-local checkmerge          = fonts.helpers.checkmerge
-local checkflags          = fonts.helpers.checkflags
-local checksteps          = fonts.helpers.checksteps
+local fonthelpers         = fonts.helpers
+local checkmerge          = fonthelpers.checkmerge
+local checkflags          = fonthelpers.checkflags
+local checksteps          = fonthelpers.checksteps
 
 local normalized = {
     substitution      = "substitution",
@@ -101,8 +102,9 @@ local function validspecification(specification,name)
         specification = { dataset = dataset }
     else
         dataset = { { data = specification.data } }
-        specification.data    = nil
-        specification.dataset = dataset
+        specification.data     = nil
+        specification.coverage = dataset
+        specification.dataset  = dataset
     end
     local first = dataset[1]
     if first then
@@ -128,7 +130,9 @@ local function validspecification(specification,name)
     end
 end
 
-local function addfeature(data,feature,specifications)
+local function addfeature(data,feature,specifications,prepareonly,filename)
+
+    -- todo: move helpers out
 
     -- todo: add some validator / check code so that we're more tolerant to
     -- user errors
@@ -140,8 +144,14 @@ local function addfeature(data,feature,specifications)
 
     local descriptions = data.descriptions
     local resources    = data.resources
-    local features     = resources.features
-    local sequences    = resources.sequences
+
+    if not descriptions or not resources then
+        report_otf("missing specification")
+        return
+    end
+
+    local features  = resources.features
+    local sequences = resources.sequences
 
     if not features or not sequences then
         report_otf("missing specification")
@@ -167,6 +177,7 @@ local function addfeature(data,feature,specifications)
     local done         = 0
     local skip         = 0
     local aglunicodes  = false
+    local privateslot  = fonthelpers.privateslot
 
     local specifications = validspecification(specifications,feature)
     if not specifications then
@@ -196,6 +207,13 @@ local function addfeature(data,feature,specifications)
                 return u
             end
         end
+        if privateslot then
+            u = privateslot(code) -- no creation !
+            if u then
+             -- unicodes[code] = u
+                return u
+            end
+        end
         local u = lpegmatch(p,code)
         if u then
          -- unicodes[code] = u
@@ -216,6 +234,9 @@ local function addfeature(data,feature,specifications)
     local stepkey      = coverup.stepkey
     local register     = coverup.register
 
+    -- todo: directly pass a coverage i.e. for privates that later will be
+    -- set
+
     local function prepare_substitution(list,featuretype,nocheck)
         local coverage = { }
         local cover    = coveractions[featuretype]
@@ -223,13 +244,14 @@ local function addfeature(data,feature,specifications)
             local unicode     = tounicode(code)
             local description = descriptions[unicode]
             if not nocheck and not description then
+                -- todo: trace !
                 skip = skip + 1
             else
                 if type(replacement) == "table" then
                     replacement = replacement[1]
                 end
                 replacement = tounicode(replacement)
-                if replacement and descriptions[replacement] then
+                if replacement and (nocheck or descriptions[replacement]) then
                     cover(coverage,unicode,replacement)
                     done = done + 1
                 else
@@ -348,14 +370,16 @@ local function addfeature(data,feature,specifications)
         data.resources .spacekerns    = nil
     end
 
-    local function prepare_kern(list,featuretype)
+    local function prepare_kern(list,featuretype,nocheck)
         local coverage = { }
         local cover    = coveractions[featuretype]
         local isspace  = false
         for code, replacement in next, list do
             local unicode     = tounicode(code)
             local description = descriptions[unicode]
-            if description and type(replacement) == "table" then
+            if not nocheck and not description then
+                skip = skip + 1
+            elseif type(replacement) == "table" then
                 local r = { }
                 for k, v in next, replacement do
                     local u = tounicode(k)
@@ -385,14 +409,16 @@ local function addfeature(data,feature,specifications)
         return coverage
     end
 
-    local function prepare_pair(list,featuretype)
+    local function prepare_pair(list,featuretype,nocheck)
         local coverage = { }
         local cover    = coveractions[featuretype]
         if cover then
             for code, replacement in next, list do
                 local unicode     = tounicode(code)
                 local description = descriptions[unicode]
-                if description and type(replacement) == "table" then
+                if not nocheck and not description then
+                    skip = skip + 1
+                elseif type(replacement) == "table" then
                     local r = { }
                     for k, v in next, replacement do
                         local u = tounicode(k)
@@ -427,13 +453,36 @@ local function addfeature(data,feature,specifications)
 
     local prepare_single = prepare_pair -- we could have a better test on the spec
 
-    local function prepare_chain(list,featuretype,sublookups)
+    local function hassteps(lookups)
+        if lookups then
+            for i=1,#lookups do
+                local l = lookups[i]
+                if l then
+                    for j=1,#l do
+                        local l = l[j]
+                        if l then
+                            local n = l.nofsteps
+                            if not n then
+                                -- gsub_remove
+                                return true
+                            elseif n > 0 then
+                                return true
+                            end
+                        end
+                    end
+                end
+            end
+        end
+        return false
+    end
+
+    -- 0 == remove, false = ignore (remove is default)
+
+    local function prepare_chain(list,featuretype,sublookups,nocheck)
         -- todo: coveractions
         local rules    = list.rules
         local coverage = { }
         if rules then
-            local rulehash   = { }
-            local rulesize   = 0
             local lookuptype = types[featuretype]
             for nofrules=1,#rules do
                 local rule         = rules[nofrules]
@@ -464,6 +513,16 @@ local function addfeature(data,feature,specifications)
                 local lookups = rule.lookups or false
                 local subtype = nil
                 if lookups and sublookups then
+                    -- inspect(lookups)
+                    if #lookups > 0 then
+                        local ns = stop - start + 1
+                        for i=1,ns do
+                            if lookups[i] == nil then
+                                lookups[i] = 0
+                            end
+                        end
+                    end
+                    local l = { }
                     for k, v in sortedhash(lookups) do
                         local t = type(v)
                         if t == "table" then
@@ -474,61 +533,76 @@ local function addfeature(data,feature,specifications)
                                     v[i] = { vi }
                                 end
                             end
+                            l[k] = v
                         elseif t == "number" then
                             local lookup = sublookups[v]
                             if lookup then
-                                lookups[k] = { lookup }
+                                l[k] = { lookup }
                                 if not subtype then
                                     subtype = lookup.type
                                 end
                             elseif v == 0 then
-                                lookups[k] = { { type = "gsub_remove" } }
+                                l[k] = { { type = "gsub_remove", nosteps = true } }
                             else
-                                lookups[k] = false -- { false } -- new
+                                l[k] = false -- { false } -- new
                             end
                         else
-                            lookups[k] = false -- { false } -- new
+                            l[k] = false -- { false } -- new
                         end
                     end
+                    if nocheck then
+                        -- fragile
+                        rule.lookups = l --no, because checking can spoil it
+                    end
+                    lookups = l
                 end
                 if nofsequences > 0 then -- we merge coverage into one
                     -- we copy as we can have different fonts
-                    local hashed = { }
-                    for i=1,nofsequences do
-                        local t = { }
-                        local s = sequence[i]
-                        for i=1,#s do
-                            local u = tounicode(s[i])
-                            if u then
-                                t[u] = true
+                    if hassteps(lookups) then
+                        -- sequence is the before|current|after match list
+                        local hashed = { }
+                        for i=1,nofsequences do
+                            local t = { }
+                            local s = sequence[i]
+                            for i=1,#s do
+                                local u = tounicode(s[i])
+                                if u then
+                                    t[u] = true
+                                end
+                            end
+                            hashed[i] = t
+                        end
+                        -- hashed is the before|current|after match hash
+                        sequence = hashed
+                        local ruleset = {
+                            nofrules,     -- 1
+                            lookuptype,   -- 2
+                            sequence,     -- 3
+                            start,        -- 4
+                            stop,         -- 5
+                            lookups,      -- 6 (6/7 also signal of what to do)
+                            replacements, -- 7
+                            subtype,      -- 8
+                        }
+                        for unic in sortedhash(sequence[start]) do
+                            local cu = coverage[unic]
+                            if cu then
+                                local n = cu.n + 1
+                                cu[n] = ruleset
+                                cu.n = n
+                            else
+                                coverage[unic] = {
+                                    ruleset,
+                                    n = 1,
+                                }
                             end
                         end
-                        hashed[i] = t
+                        sequence.n = nofsequences
+                    else
+                     -- report_otf("no steps for %a",lookuptype) -- e.g. in primes feature
                     end
-                    sequence = hashed
-                    -- now we create the rule
-                    rulesize = rulesize + 1
-                    rulehash[rulesize] = {
-                        nofrules,     -- 1
-                        lookuptype,   -- 2
-                        sequence,     -- 3
-                        start,        -- 4
-                        stop,         -- 5
-                        lookups,      -- 6 (6/7 also signal of what to do)
-                        replacements, -- 7
-                        subtype,      -- 8
-                    }
---                     for unic in next, sequence[start] do
-                    for unic in sortedhash(sequence[start]) do
-                        local cu = coverage[unic]
-                        if not cu then
-                            coverage[unic] = rulehash -- can now be done cleaner i think
-                        end
-                    end
-                    sequence.n = nofsequences
                 end
             end
-            rulehash.n = rulesize
         end
         return coverage
     end
@@ -606,6 +680,29 @@ local function addfeature(data,feature,specifications)
     for s=1,#dataset do
         local specification = dataset[s]
         local valid = specification.valid -- nowhere used
+        local files = specification.files
+        if files and filename then
+            local base = string.lower(file.basename(filename))
+            -- hash test
+            local okay = files[base]
+            -- list test
+            if not okay then
+                for i=1,#files do
+                    if base == files[i] then
+                        okay = true
+                        break
+                    end
+                end
+            end
+            if okay then
+             -- report_otf("feature %a applied to file %a",specification.name,base)
+            else
+             -- report_otf("feature %a skipped for file %a",specification.name,base)
+             -- return
+                goto next
+            end
+        end
+        --
         local feature = specification.name or feature
         if not feature or feature == "" then
             report_otf("no valid name given for extra feature")
@@ -617,134 +714,172 @@ local function addfeature(data,feature,specifications)
             end
             local askedfeatures = specification.features or everywhere
             local askedsteps    = specification.steps or specification.subtables or { specification.data } or { }
-            local featuretype   = normalized[specification.type or "substitution"] or "substitution"
+            local featuretype   = specification.type or "substitution"
+            local featureaction = false
             local featureflags  = specification.flags or noflags
             local nocheck       = specification.nocheck
-            local futuresteps   = specification.futuresteps
+            local mapping       = specification.mapping
             local featureorder  = specification.order or { feature }
             local featurechain  = (featuretype == "chainsubstitution" or featuretype == "chainposition") and 1 or 0
             local nofsteps      = 0
             local steps         = { }
             local sublookups    = specification.lookups
             local category      = nil
+            local steptype      = nil
+            local sequence      = nil
+            --
+            if fonts.handlers.otf.handlers[featuretype] then
+                featureaction = true -- function based
+            else
+                featuretype = normalized[specification.type or "substitution"] or "substitution"
+            end
             --
             checkflags(specification,resources)
             --
-            if sublookups then
-                local s = { }
-                for i=1,#sublookups do
-                    local specification = sublookups[i]
-                    local askedsteps    = specification.steps or specification.subtables or { specification.data } or { }
-                    local featuretype   = normalized[specification.type or "substitution"] or "substitution"
-                    local featureflags  = specification.flags or noflags
-                    local nofsteps      = 0
-                    local steps         = { }
-                    for i=1,#askedsteps do
-                        local list     = askedsteps[i]
-                        local coverage = nil
-                        local format   = nil
-                        if featuretype == "substitution" then
-                            coverage = prepare_substitution(list,featuretype,nocheck)
-                        elseif featuretype == "ligature" then
-                            coverage = prepare_ligature(list,featuretype,nocheck)
-                        elseif featuretype == "alternate" then
-                            coverage = prepare_alternate(list,featuretype,nocheck)
-                        elseif featuretype == "multiple" then
-                            coverage = prepare_multiple(list,featuretype,nocheck)
-                        elseif featuretype == "kern" or featuretype == "move" then
-                            format   = featuretype
-                            coverage = prepare_kern(list,featuretype)
-                        elseif featuretype == "pair" then
-                            format   = "pair"
-                            coverage = prepare_pair(list,featuretype)
-                        elseif featuretype == "single" then
-                            format   = "single"
-                            coverage = prepare_single(list,featuretype)
-                        end
-                        if coverage and next(coverage) then
-                            nofsteps = nofsteps + 1
-                            steps[nofsteps] = register(coverage,featuretype,format,feature,nofsteps,descriptions,resources)
-                        end
-                    end
-                    --
-                    checkmerge(specification)
-                    checksteps(specification)
-                    --
-                    s[i] = {
-                        [stepkey] = steps,
-                        nofsteps  = nofsteps,
-                        flags     = featureflags,
-                        type      = types[featuretype],
-                    }
-                end
-                sublookups = s
-            end
-            for i=1,#askedsteps do
-                local list     = askedsteps[i]
-                local coverage = nil
-                local format   = nil
-                if featuretype == "substitution" then
-                    category = "gsub"
-                    coverage = prepare_substitution(list,featuretype,nocheck)
-                elseif featuretype == "ligature" then
-                    category = "gsub"
-                    coverage = prepare_ligature(list,featuretype,nocheck)
-                elseif featuretype == "alternate" then
-                    category = "gsub"
-                    coverage = prepare_alternate(list,featuretype,nocheck)
-                elseif featuretype == "multiple" then
-                    category = "gsub"
-                    coverage = prepare_multiple(list,featuretype,nocheck)
-                elseif featuretype == "kern" or featuretype == "move" then
-                    category = "gpos"
-                    format   = featuretype
-                    coverage = prepare_kern(list,featuretype)
-                elseif featuretype == "pair" then
-                    category = "gpos"
-                    format   = "pair"
-                    coverage = prepare_pair(list,featuretype)
-                elseif featuretype == "single" then
-                    category = "gpos"
-                    format   = "single"
-                    coverage = prepare_single(list,featuretype)
-                elseif featuretype == "chainsubstitution" then
-                    category = "gsub"
-                    coverage = prepare_chain(list,featuretype,sublookups)
-                elseif featuretype == "chainposition" then
-                    category = "gpos"
-                    coverage = prepare_chain(list,featuretype,sublookups)
-                else
-                    report_otf("not registering feature %a, unknown category",feature)
-                    return
-                end
-                if coverage and next(coverage) then
-                    nofsteps = nofsteps + 1
-                    steps[nofsteps] = register(coverage,featuretype,format,feature,nofsteps,descriptions,resources)
+            for k, v in next, askedfeatures do
+                if v[1] then
+                    askedfeatures[k] = tohash(v)
                 end
             end
-            if nofsteps > 0 then
-                -- script = { lang1, lang2, lang3 } or script = { lang1 = true, ... }
-                for k, v in next, askedfeatures do
-                    if v[1] then
-                        askedfeatures[k] = tohash(v)
-                    end
-                end
-                --
-                if featureflags[1] then featureflags[1] = "mark" end
-                if featureflags[2] then featureflags[2] = "ligature" end
-                if featureflags[3] then featureflags[3] = "base" end
-                local steptype = types[featuretype]
-                local sequence = {
-                    chain     = featurechain,
+            --
+            if featureflags[1] then featureflags[1] = "mark" end
+            if featureflags[2] then featureflags[2] = "ligature" end
+            if featureflags[3] then featureflags[3] = "base" end
+            --
+            if featureaction then
+
+                category = "gsub"
+                sequence = {
                     features  = { [feature] = askedfeatures },
                     flags     = featureflags,
                     name      = feature, -- redundant
                     order     = featureorder,
-                    [stepkey] = steps,
-                    nofsteps  = nofsteps,
-                    type      = steptype,
+                    type      = featuretype,
+                 -- steps     = { },
+                    nofsteps  = 0, -- just in case we test for that
                 }
-                --
+
+            else
+
+                if sublookups then
+                    local s = { }
+                    for i=1,#sublookups do
+                        local specification = sublookups[i]
+                        local askedsteps    = specification.steps or specification.subtables or { specification.data } or { }
+                        local featuretype   = normalized[specification.type or "substitution"] or "substitution"
+                        local featureflags  = specification.flags or noflags
+                        local nofsteps      = 0
+                        local steps         = { }
+                        for i=1,#askedsteps do
+                            local list     = askedsteps[i]
+                            local coverage = nil
+                            local format   = nil
+                            if featuretype == "substitution" then
+                                coverage = prepare_substitution(list,featuretype,nocheck)
+                            elseif featuretype == "ligature" then
+                                coverage = prepare_ligature(list,featuretype,nocheck)
+                            elseif featuretype == "alternate" then
+                                coverage = prepare_alternate(list,featuretype,nocheck)
+                            elseif featuretype == "multiple" then
+                                coverage = prepare_multiple(list,featuretype,nocheck)
+                            elseif featuretype == "kern" or featuretype == "move" then
+                                format   = featuretype
+                                coverage = prepare_kern(list,featuretype,nocheck)
+                            elseif featuretype == "pair" then
+                                format   = "pair"
+                                coverage = prepare_pair(list,featuretype,nocheck)
+                            elseif featuretype == "single" then
+                                format   = "single"
+                                coverage = prepare_single(list,featuretype,nocheck)
+                            end
+                            if coverage and next(coverage) then
+                                nofsteps = nofsteps + 1
+                                steps[nofsteps] = register(coverage,featuretype,format,feature,nofsteps,descriptions,resources)
+                            end
+                        end
+                        --
+                        checkmerge(specification)
+                        checksteps(specification)
+                        --
+                        s[i] = {
+                            [stepkey] = steps,
+                            nofsteps  = nofsteps,
+                            flags     = featureflags,
+                            type      = types[featuretype],
+                        }
+                    end
+                    sublookups = s
+                end
+
+                for i=1,#askedsteps do
+                    local list     = askedsteps[i]
+                    local coverage = nil
+                    local format   = nil
+                    if type(list) == "function" then
+                        list = list(data,specification,list,i) -- why pass list instead if askedsteps
+                    end
+                    if not list then
+                        -- see ebgaramond hack
+                    elseif featuretype == "substitution" then
+                        -- see font-imp-tweaks: we directly pass a mapping so no checks done
+                        category = "gsub"
+                        coverage = (mapping and list) or prepare_substitution(list,featuretype,nocheck)
+                    elseif featuretype == "ligature" then
+                        category = "gsub"
+                        coverage = prepare_ligature(list,featuretype,nocheck)
+                    elseif featuretype == "alternate" then
+                        category = "gsub"
+                        coverage = prepare_alternate(list,featuretype,nocheck)
+                    elseif featuretype == "multiple" then
+                        category = "gsub"
+                        coverage = prepare_multiple(list,featuretype,nocheck)
+                    elseif featuretype == "kern" or featuretype == "move" then
+                        category = "gpos"
+                        format   = featuretype
+                        coverage = prepare_kern(list,featuretype,nocheck)
+                    elseif featuretype == "pair" then
+                        category = "gpos"
+                        format   = "pair"
+                        coverage = prepare_pair(list,featuretype,nocheck)
+                    elseif featuretype == "single" then
+                        category = "gpos"
+                        format   = "single"
+                        coverage = prepare_single(list,featuretype,nocheck)
+                    elseif featuretype == "chainsubstitution" then
+                        category = "gsub"
+                        coverage = prepare_chain(list,featuretype,sublookups,nocheck)
+                    elseif featuretype == "chainposition" then
+                        category = "gpos"
+                        coverage = prepare_chain(list,featuretype,sublookups,nocheck)
+                    else
+                        report_otf("not registering feature %a, unknown category",feature)
+                        return
+                    end
+                    if coverage and next(coverage) then
+                        nofsteps = nofsteps + 1
+                        steps[nofsteps] = register(coverage,featuretype,format,feature,nofsteps,descriptions,resources)
+                    end
+                end
+
+                if nofsteps > 0 then
+                    sequence = {
+                        chain     = featurechain,
+                        features  = { [feature] = askedfeatures },
+                        flags     = featureflags,
+                        name      = feature, -- redundant
+                        order     = featureorder,
+                        [stepkey] = steps,
+                        nofsteps  = nofsteps,
+                        type      = specification.handler or types[featuretype],
+                    }
+                    if prepareonly then
+                        return sequence
+                    end
+                end
+            end
+
+            if sequence then
+                -- script = { lang1, lang2, lang3 } or script = { lang1 = true, ... }
                 checkflags(sequence,resources)
                 checkmerge(sequence)
                 checksteps(sequence)
@@ -774,11 +909,14 @@ local function addfeature(data,feature,specifications)
                     end
                 end
             end
+
         end
+      ::next::
     end
     if trace_loading then
         report_otf("registering feature %a, affected glyphs %a, skipped glyphs %a",feature,done,skip)
     end
+
 end
 
 otf.enhancers.addfeature = addfeature
@@ -802,7 +940,7 @@ function otf.addfeature(name,specification)
             slot = #extrafeatures + 1
             knownfeatures[name] = slot
         elseif specification.overload == false then
-            -- we add an extre one
+            -- we add an extra one
             slot = #extrafeatures + 1
             knownfeatures[name] = slot
         else
@@ -821,10 +959,27 @@ end
 local function enhance(data,filename,raw)
     for slot=1,#extrafeatures do
         local specification = extrafeatures[slot]
-        addfeature(data,specification.name,specification)
+        addfeature(data,specification.name,specification,nil,filename)
     end
 end
 
+-- local function enhance(data,filename,raw)
+--     local first = 1
+--     local last  = #extrafeatures
+--     while true do
+--         for slot=first,last do
+--             local specification = extrafeatures[slot]
+--             addfeature(data,specification.name,specification)
+--         end
+--         if #extrafeatures > last then
+--             first = last + 1
+--             last  = #extrafeatures
+--         else
+--             break
+--         end
+--     end
+-- end
+
 otf.enhancers.enhance = enhance
 
 otf.enhancers.register("check extra features",enhance)
