@@ -26,7 +26,7 @@ if not modules then modules = { } end modules ['font-otl'] = {
 local lower = string.lower
 local type, next, tonumber, tostring, unpack = type, next, tonumber, tostring, unpack
 local abs = math.abs
-local derivetable, sortedhash = table.derive, table.sortedhash
+local derivetable, sortedhash, remove = table.derive, table.sortedhash, table.remove
 local formatters = string.formatters
 
 local setmetatableindex   = table.setmetatableindex
@@ -52,7 +52,7 @@ local report_otf          = logs.reporter("fonts","otf loading")
 local fonts               = fonts
 local otf                 = fonts.handlers.otf
 
-otf.version               = 3.133 -- beware: also sync font-mis.lua and in mtx-fonts
+otf.version               = 3.145 -- beware: also sync font-mis.lua and in mtx-fonts
 otf.cache                 = containers.define("fonts", "otl", otf.version, true)
 otf.svgcache              = containers.define("fonts", "svg", otf.version, true)
 otf.pngcache              = containers.define("fonts", "png", otf.version, true)
@@ -311,13 +311,13 @@ end
 -- instance protruding info and loop over characters; one is not supposed
 -- to change descriptions and if one does so one should make a copy!
 
-local function copytotfm(data,cache_id)
+local function copytotfm(data,cache_id,wipemath)
     if data then
         local metadata       = data.metadata
         local properties     = derivetable(data.properties)
         local descriptions   = derivetable(data.descriptions)
         local goodies        = derivetable(data.goodies)
-        local characters     = { } -- newtable if we knwo how many
+        local characters     = { } -- newtable if we know how many
         local parameters     = { }
         local mathparameters = { }
         --
@@ -335,15 +335,70 @@ local function copytotfm(data,cache_id)
             minsize    = 100
             maxsize    = 100
         end
-        if mathspecs then
-            for name, value in next, mathspecs do
-                mathparameters[name] = value
-            end
-        end
+        --
         for unicode in next, data.descriptions do -- use parent table
             characters[unicode] = { }
         end
-        if mathspecs then
+        --
+        -- we need a runtime lookup because of running from cdrom or zip, brrr (shouldn't
+        -- we use the basename then?)
+        --
+        local filename = constructors.checkedfilename(resources)
+        local fontname = metadata.fontname
+        local fullname = metadata.fullname or fontname
+        local psname   = fontname or fullname
+        local subfont  = metadata.subfontindex
+        local units    = metadata.units or 1000
+        --
+        if units == 0 then -- catch bugs in fonts
+            units = 1000 -- maybe 2000 when ttf
+            metadata.units = 1000
+            report_otf("changing %a units to %a",0,units)
+        end
+        --
+        if not mathspecs then
+            -- go on
+        elseif wipemath then
+            -- No way that we let a non (or partial) math font register itself as math and interfere
+            -- in bad ways (apart from unwanted overhead). I've seen some examples and it proofs how
+            -- instable and unpredictable fonts can become.
+            local wiped     = false
+            local features  = resources.features
+            local sequences = resources.sequences
+            if features then
+                local gsub = features.gsub
+                if gsub and gsub.ssty then
+                    gsub.ssty = nil
+                    wiped = true
+                end
+            end
+            if sequences then
+                for i=#sequences,1,-1 do
+                    local sequence = sequences[i]
+                    local features = sequence.features
+                    if features then
+                        if features.ssty then
+                            remove(sequences,i)
+                            wiped = true
+                        end
+                    end
+                end
+            end
+            if resources.mathconstants then
+                resources.mathconstants = nil
+                wiped = true
+            end
+            if resources.math then
+                metadata.math = nil
+                wiped = true
+            end
+            if wiped then
+                report_otf("math data wiped from %a",fullname)
+            end
+        else
+            for name, value in next, mathspecs do
+                mathparameters[name] = value
+            end
             for unicode, character in next, characters do
                 local d = descriptions[unicode] -- we could use parent table here
                 local m = d.math
@@ -404,20 +459,6 @@ local function copytotfm(data,cache_id)
                 end
             end
         end
-        -- we need a runtime lookup because of running from cdrom or zip, brrr (shouldn't
-        -- we use the basename then?)
-        local filename = constructors.checkedfilename(resources)
-        local fontname = metadata.fontname
-        local fullname = metadata.fullname or fontname
-        local psname   = fontname or fullname
-        local subfont  = metadata.subfontindex
-        local units    = metadata.units or 1000
-        --
-        if units == 0 then -- catch bugs in fonts
-            units = 1000 -- maybe 2000 when ttf
-            metadata.units = 1000
-            report_otf("changing %a units to %a",0,units)
-        end
         --
         local monospaced  = metadata.monospaced
         local charwidth   = metadata.averagewidth -- or unset
@@ -497,43 +538,17 @@ local function copytotfm(data,cache_id)
         parameters.units      = units
         parameters.vheight    = metadata.defaultvheight
         --
-        properties.space      = spacer
-        properties.format     = data.format or formats.otf
-        properties.filename   = filename
-        properties.fontname   = fontname
-        properties.fullname   = fullname
-        properties.psname     = psname
-        properties.name       = filename or fullname
-        properties.subfont    = subfont
-        --
-if not CONTEXTLMTXMODE or CONTEXTLMTXMODE == 0 then
-    --
-    properties.encodingbytes = 2
-elseif CONTEXTLMTXMODE then
-    local duplicates = resources and resources.duplicates
-    if duplicates then
-        local maxindex = data.nofglyphs or metadata.nofglyphs
-        if maxindex then
-            for u, d in sortedhash(duplicates) do
-                local du = descriptions[u]
-                if du then
-                    for uu in sortedhash(d) do
-                        maxindex = maxindex + 1
-                        descriptions[uu].dupindex = du.index
-                        descriptions[uu].index    = maxindex
-                    end
-                else
-                 -- report_otf("no %U in font %a, duplicates ignored",u,filename)
-                end
-            end
-        end
-    end
-    --
-end
-        --
+        properties.space         = spacer
+        properties.format        = data.format or formats.otf
+        properties.filename      = filename
+        properties.fontname      = fontname
+        properties.fullname      = fullname
+        properties.psname        = psname
+        properties.name          = filename or fullname
+        properties.subfont       = subfont
+        properties.encodingbytes = 2
      -- properties.name          = specification.name
      -- properties.sub           = specification.sub
-        --
         properties.private       = properties.private or data.private or privateoffset
         --
         return {
@@ -590,9 +605,7 @@ local function otftotfm(specification)
     local cache_id = specification.hash
     local tfmdata  = containers.read(constructors.cache,cache_id)
     if not tfmdata then
-
         checkconversion(specification) -- for the moment here
-
         local name     = specification.name
         local sub      = specification.sub
         local subindex = specification.subindex
@@ -603,7 +616,7 @@ local function otftotfm(specification)
         if rawdata and next(rawdata) then
             local descriptions = rawdata.descriptions
             rawdata.lookuphash = { } -- to be done
-            tfmdata = copytotfm(rawdata,cache_id)
+            tfmdata = copytotfm(rawdata,cache_id,features and features.wipemath)
             if tfmdata and next(tfmdata) then
                 -- at this moment no characters are assigned yet, only empty slots
                 local features     = constructors.checkedfeatures("otf",features)
