@@ -262,10 +262,8 @@ local function preparesubstitutions(tfmdata,feature,value,validlookups,lookuplis
         if kind == "gsub_single" then
             for i=1,#steps do
                 for unicode, data in next, steps[i].coverage do
-                    if unicode ~= data then
--- if not changed[unicode] then
+                    if unicode ~= data and not changed[unicode] then
                         changed[unicode] = data
--- end
                     end
                     if trace_singles then
                         report_substitution(feature,sequence,descriptions,unicode,data)
@@ -277,30 +275,24 @@ local function preparesubstitutions(tfmdata,feature,value,validlookups,lookuplis
                 for unicode, data in next, steps[i].coverage do
                     local replacement = data[alternate]
                     if replacement then
-                        if unicode ~= replacement then
--- if not changed[unicode] then
+                        if unicode ~= replacement and not changed[unicode] then
                             changed[unicode] = replacement
--- end
                         end
                         if trace_alternatives then
                             report_alternate(feature,sequence,descriptions,unicode,replacement,value,"normal")
                         end
                     elseif defaultalt == "first" then
                         replacement = data[1]
-                        if unicode ~= replacement then
--- if not changed[unicode] then
+                        if unicode ~= replacement and not changed[unicode] then
                             changed[unicode] = replacement
--- end
                         end
                         if trace_alternatives then
                             report_alternate(feature,sequence,descriptions,unicode,replacement,value,defaultalt)
                         end
                     elseif defaultalt == "last" then
                         replacement = data[#data]
-                        if unicode ~= replacement then
--- if not changed[unicode] then
+                        if unicode ~= replacement and not changed[unicode] then
                             changed[unicode] = replacement
--- end
                         end
                         if trace_alternatives then
                             report_alternate(feature,sequence,descriptions,unicode,replacement,value,defaultalt)
