@@ -6,11 +6,12 @@ if not modules then modules = { } end modules ['lang-wrd'] = {
     license   = "see context related readme files"
 }
 
+local next, tonumber = next, tonumber
 local lower = string.lower
 local utfchar = utf.char
 local concat, setmetatableindex = table.concat, table.setmetatableindex
 local lpegmatch = lpeg.match
-local P, S, Cs, Cf, Cg, Cc, C = lpeg.P, lpeg.S, lpeg.Cs, lpeg.Cf, lpeg.Cg, lpeg.Cc, lpeg.C
+local P, S, Cs, Cc, C = lpeg.P, lpeg.S, lpeg.Cs, lpeg.Cc, lpeg.C
 
 local report_words = logs.reporter("languages","words")
 
@@ -30,19 +31,19 @@ local numbers         = languages.numbers
 local registered      = languages.registered
 
 local nuts            = nodes.nuts
-local tonut           = nuts.tonut
 
-local getfield        = nuts.getfield
+----- getfield        = nuts.getfield
 local getnext         = nuts.getnext
 local getid           = nuts.getid
-local getsubtype      = nuts.getsubtype
+----- getsubtype      = nuts.getsubtype
 local getchar         = nuts.getchar
 local setattr         = nuts.setattr
-local getlang         = nuts.getlang
-local isglyph         = nuts.isglyph
+----- getattr         = nuts.getattr
+local getlanguage     = nuts.getlanguage
+local ischar          = nuts.ischar
 
-local traverse_nodes  = nuts.traverse
-local traverse_ids    = nuts.traverse_id
+local nextnode        = nuts.traversers.node
+----- nextglyph       = nuts.traversers.glyph
 
 local wordsdata       = words.data
 local chardata        = characters.data
@@ -51,13 +52,14 @@ local enableaction    = nodes.tasks.enableaction
 local unsetvalue      = attributes.unsetvalue
 
 local nodecodes       = nodes.nodecodes
-local kerncodes       = nodes.kerncodes
+----- kerncodes       = nodes.kerncodes
 
 local glyph_code      = nodecodes.glyph
-local disc_code       = nodecodes.disc
-local kern_code       = nodecodes.kern
+----- disc_code       = nodecodes.disc
+----- kern_code       = nodecodes.kern
+
+----- fontkern_code   = kerncodes.fontkern
 
-local kerning_code    = kerncodes.kerning
 local lowerchar       = characters.lower
 
 local a_color         = attributes.private('color')
@@ -83,7 +85,6 @@ local loaders = {
         local data = io.loaddata(fullname)
         if data and data ~= "" then
             local parser = (spacing + word/function(s) list[s] = true end)^0
-         -- local parser = Cf(Cc(list) * Cg(spacing^0 * word * Cc(true))^1,rawset) -- not better
             lpegmatch(parser,data)
         end
     end,
@@ -142,14 +143,13 @@ end
 -- there is an n=1 problem somewhere in nested boxes
 
 local function mark_words(head,whenfound) -- can be optimized and shared
-    local current, language, done = tonut(head), nil, nil, 0, false
+    local current, language = head, nil, nil, 0
     local str, s, nds, n = { }, 0, { }, 0 -- n could also be a table, saves calls
     local function action()
         if s > 0 then
             local word = concat(str,"",1,s)
             local mark = whenfound(language,word)
             if mark then
-                done = true
                 for i=1,n do
                     mark(nds[i])
                 end
@@ -160,9 +160,9 @@ local function mark_words(head,whenfound) -- can be optimized and shared
     -- we haven't done the fonts yet so we have characters (otherwise
     -- we'd have to use the tounicodes)
     while current do
-        local code, id = isglyph(current)
-        if code then
-            local a = getlang(current)
+        local code, id = ischar(current) -- not isglyph because otherwise we can run into
+        if code then                     -- processed streams (\about[foo] does that)
+            local a = getlanguage(current)
             if a then
                 if a ~= language then
                     if s > 0 then
@@ -174,6 +174,7 @@ local function mark_words(head,whenfound) -- can be optimized and shared
                 action()
                 language = a
             end
+            -- still unicode
             local data = chardata[code]
             if is_letter[data.category] then
                 n = n + 1
@@ -183,21 +184,29 @@ local function mark_words(head,whenfound) -- can be optimized and shared
             elseif s > 0 then
                 action()
             end
-        elseif id == disc_code then -- take the replace
-            if n > 0 then
-                local r = getfield(current,"replace")
-                if r then
-                    for current in traverse_ids(glyph_code,r) do
-                        local code = getchar(current)
-                        n = n + 1
-                        nds[n] = current
-                        s = s + 1
-                        str[s] = utfchar(code)
-                    end
-                end
-            end
-        elseif id == kern_code and getsubtype(current) == kerning_code and s > 0 then
-            -- ok
+     -- elseif id == disc_code then
+     --     -- take the replace .. we kick in before we hyphenate so we're
+     --     -- not yet seeing many discs and we only handle explicit ones
+     --     -- in fact we could as well decide to ignore words with a disc
+     --     -- because we then have a compound word
+     --     if n > 0 then
+     --         local r = getfield(current,"replace")
+     --         if r then
+     --             -- also disc itself
+     --             n = n + 1
+     --             nds[n] = current
+     --             --
+     --             for current in nextglyph, r do
+     --                 local code = getchar(current)
+     --                 n = n + 1
+     --                 nds[n] = current
+     --                 s = s + 1
+     --                 str[s] = utfchar(code)
+     --             end
+     --         end
+     --     end
+     -- elseif id == kern_code and getsubtype(current) == fontkern_code and s > 0 then
+     --     -- ok
         elseif s > 0 then
             action()
         end
@@ -206,7 +215,7 @@ local function mark_words(head,whenfound) -- can be optimized and shared
     if s > 0 then
         action()
     end
-    return head, done
+    return head
 end
 
 local methods  = { }
@@ -221,8 +230,8 @@ local enabled    = false
 function words.check(head)
     if enabled then
         return methods[wordmethod](head)
-    elseif not head then
-        return head, false
+ -- elseif not head then
+ --     return head, false
     else
         return head, false
     end
@@ -274,7 +283,7 @@ local function sweep(language,str)
 end
 
 methods[1] = function(head)
-    for n in traverse_nodes(head) do
+    for n in nextnode, head do
         setattr(n,a_color,unsetvalue) -- hm, not that selective (reset color)
     end
     return mark_words(head,sweep)
@@ -369,7 +378,7 @@ local function sweep(language,str)
 end
 
 methods[3] = function(head)
-    for n in traverse_nodes(head) do
+    for n in nextnode, head do
         setattr(n,a_color,unsetvalue)
     end
     return mark_words(head,sweep)
@@ -409,6 +418,6 @@ implement {
 
 implement {
     name      = "loadspellchecklist",
-    arguments = { "string", "string" },
+    arguments = "2 strings",
     actions   = words.load
 }
