@@ -23,9 +23,14 @@ if not modules then modules = { } end modules ['lpdf-wid'] = {
 -- the other fragile bit. And, now that adobe quits flash in 2020 we're without any
 -- video anyway. Also, it won't play on all platforms and devices so let's wait for
 -- html5 media in pdf then.
+--
+-- See mail by Michal Vlasák to the mailing list that discusses current support in
+-- viewers and also mentions (and submitted) a few fixes wrt embedding media.
 
 local tonumber, next = tonumber, next
 local gmatch, gsub, find, lower = string.gmatch, string.gsub, string.find, string.lower
+local filenameonly, basefilename, filesuffix, addfilesuffix = file.nameonly, file.basename, file.suffix, file.addsuffix
+local isfile, modificationtime = lfs.isfile, lfs.modification
 local stripstring = string.strip
 local settings_to_array = utilities.parsers.settings_to_array
 local settings_to_hash = utilities.parsers.settings_to_hash
@@ -41,6 +46,8 @@ local context                  = context
 
 local texgetcount              = tex.getcount
 
+local hpacknode                = nodes.hpack
+
 local nodeinjections           = backends.pdf.nodeinjections
 local codeinjections           = backends.pdf.codeinjections
 local registrations            = backends.pdf.registrations
@@ -53,28 +60,28 @@ local v_auto                   = variables.auto
 local v_embed                  = variables.embed
 local v_max                    = variables.max
 local v_yes                    = variables.yes
+local v_compress               = variables.compress
 
 local pdfconstant              = lpdf.constant
+local pdfnull                  = lpdf.null
 local pdfdictionary            = lpdf.dictionary
 local pdfarray                 = lpdf.array
 local pdfreference             = lpdf.reference
 local pdfunicode               = lpdf.unicode
 local pdfstring                = lpdf.string
 local pdfboolean               = lpdf.boolean
-local pdfflushobject           = lpdf.flushobject
-local pdfflushstreamobject     = lpdf.flushstreamobject
-local pdfflushstreamfileobject = lpdf.flushstreamfileobject
-local pdfreserveobject         = lpdf.reserveobject
-local pdfpagereference         = lpdf.pagereference
-local pdfshareobjectreference  = lpdf.shareobjectreference
 local pdfaction                = lpdf.action
 local pdfborder                = lpdf.border
 
 local pdftransparencyvalue     = lpdf.transparencyvalue
 local pdfcolorvalues           = lpdf.colorvalues
 
-local hpack_node               = node.hpack
-local write_node               = node.write -- test context(...) instead
+local pdfflushobject           = lpdf.flushobject
+local pdfflushstreamobject     = lpdf.flushstreamobject
+local pdfflushstreamfileobject = lpdf.flushstreamfileobject
+local pdfreserveobject         = lpdf.reserveobject
+local pdfpagereference         = lpdf.pagereference
+local pdfshareobjectreference  = lpdf.shareobjectreference
 
 -- symbols
 
@@ -222,6 +229,15 @@ local referenced        = { }
 local ignorereferenced  = true -- fuzzy pdf spec .. twice in attachment list, can become an option
 local tobesavedobjrefs  = utilities.storage.allocate()
 local collectedobjrefs  = utilities.storage.allocate()
+local permitted         = true
+local enabled           = true
+
+function codeinjections.setattachmentsupport(option)
+    if option == false then
+        permitted = false
+        enabled   = false
+    end
+end
 
 local fileobjreferences = {
     collected = collectedobjrefs,
@@ -238,192 +254,218 @@ end
 job.register('job.fileobjreferences.collected', tobesavedobjrefs, initializer)
 
 local function flushembeddedfiles()
-    if next(filestreams) then
+    if enabled and next(filestreams) then
         local e = pdfarray()
+        local f = pdfarray()
         for tag, reference in sortedhash(filestreams) do
             if not reference then
                 report_attachment("unreferenced file, tag %a",tag)
-            elseif referenced[tag] == "hidden" then
+            elseif referenced[tag] == "hidden" or referenced[tag] == "forced" then
                 e[#e+1] = pdfstring(tag)
                 e[#e+1] = reference -- already a reference
+                f[#f+1] = reference -- collect all file description references
             else
-         --     -- messy spec ... when annot not in named else twice in menu list acrobat
+                -- messy spec ... when annot not in named else twice in menu list acrobat
+                f[#f+1] = reference
             end
         end
         if #e > 0 then
             lpdf.addtonames("EmbeddedFiles",pdfreference(pdfflushobject(pdfdictionary{ Names = e })))
         end
+        if #f > 0 then -- PDF/A-2|3: all associated files must have a relationship to the PDF document (global or part)
+            lpdf.addtocatalog("AF", pdfreference(pdfflushobject(f))) -- global (Catalog)
+        end
     end
 end
 
 lpdf.registerdocumentfinalizer(flushembeddedfiles,"embeddedfiles")
 
 function codeinjections.embedfile(specification)
-    local data     = specification.data
-    local filename = specification.file
-    local name     = specification.name or ""
-    local title    = specification.title or ""
-    local hash     = specification.hash or filename
-    local keepdir  = specification.keepdir -- can change
-    local usedname = specification.usedname
-    local filetype = specification.filetype
-    local compress = specification.compress
-    if filename == "" then
-        filename = nil
-    end
-    if compress == nil then
-        compress = true
-    end
-    if data then
-        local r = filestreams[hash]
-        if r == false then
-            return nil
-        elseif r then
-            return r
-        elseif not filename then
-            filename = specification.tag
-            if not filename or filename == "" then
-                filename = specification.registered
-            end
-            if not filename or filename == "" then
-                filename = hash
-            end
+    if enabled then
+        local data      = specification.data
+        local filename  = specification.file
+        local name      = specification.name or ""
+        local title     = specification.title or ""
+        local hash      = specification.hash or filename
+        local keepdir   = specification.keepdir -- can change
+        local usedname  = specification.usedname
+        local filetype  = specification.filetype
+        local compress  = specification.compress
+        local mimetype  = specification.mimetype or specification.mime
+        if filename == "" then
+            filename = nil
         end
-    else
-        if not filename then
-            return nil
+        if compress == nil then
+            compress = true
         end
-        local r = filestreams[hash]
-        if r == false then
-            return nil
-        elseif r then
-            return r
+        if data then
+            local r = filestreams[hash]
+            if r == false then
+                return nil
+            elseif r then
+                return r
+            elseif not filename then
+                filename = specification.tag
+                if not filename or filename == "" then
+                    filename = specification.registered
+                end
+                if not filename or filename == "" then
+                    filename = hash
+                end
+            end
         else
+            if not filename then
+                return nil
+            end
+            local r = filestreams[hash]
+            if r == false then
+                return nil
+            elseif r then
+                return r
+            else
+                local foundname = resolvers.findbinfile(filename) or ""
+                if foundname == "" or not isfile(foundname) then
+                    filestreams[filename] = false
+                    return nil
+                else
+                    specification.foundname = foundname
+                end
+            end
+        end
+        -- needs to be cleaned up:
+        usedname = usedname ~= "" and usedname or filename or name
+        local basename  = keepdir == true and usedname or basefilename(usedname)
+        local basename  = gsub(basename,"%./","")
+        local savename  = name ~= "" and name or basename
+        local foundname = specification.foundname or filename
+        if not filetype or filetype == "" then
+            filetype = name and (filename and filesuffix(filename)) or "txt"
+        end
+        savename = addfilesuffix(savename,filetype) -- type is mandate for proper working in viewer
+        local a = pdfdictionary {
+            Type    = pdfconstant("EmbeddedFile"),
+            Subtype = mimetype and mimetype ~= "" and pdfconstant(mimetype) or nil,
+        }
+        local f
+        if data then
+            f = pdfflushstreamobject(data,a)
+            specification.data = true -- signal that still data but already flushed
+        else
+            local attributes   = lfs.attributes(foundname)
+            local modification = modificationtime(foundname)
+            a.Params = {
+                Size    = attributes.size,
+                ModDate = lpdf.pdftimestamp(modification),
+            }
+            f = pdfflushstreamfileobject(foundname,a,compress)
+        end
+        local d = pdfdictionary {
+            Type           = pdfconstant("Filespec"),
+            F              = pdfstring(savename),
+         -- UF             = pdfstring(savename),
+            UF             = pdfunicode(savename),
+            EF             = pdfdictionary { F = pdfreference(f) },
+            Desc           = title ~= "" and pdfunicode(title) or nil,
+            AFRelationship = pdfconstant("Unspecified"), -- Supplement, Data, Source, Alternative, Data
+        }
+        local r = pdfreference(pdfflushobject(d))
+        filestreams[hash] = r
+        if specification.forcereference == true then
+            referenced[hash] = "forced"
+        end
+        return r
+    end
+end
+
+function nodeinjections.attachfile(specification)
+    if enabled then
+        local registered = specification.registered or "<unset>"
+        local data = specification.data
+        local hash
+        local filename
+        if data then
+            hash = md5.HEX(data)
+        else
+            filename = specification.file
+            if not filename or filename == "" then
+                report_attachment("no file specified, using registered %a instead",registered)
+                filename = registered
+                specification.file = registered
+            end
             local foundname = resolvers.findbinfile(filename) or ""
-            if foundname == "" or not lfs.isfile(foundname) then
-                filestreams[filename] = false
+            if foundname == "" or not isfile(foundname) then
+                report_attachment("invalid filename %a, ignoring registered %a",filename,registered)
                 return nil
             else
                 specification.foundname = foundname
             end
+            hash = filename
         end
-    end
-    -- needs to cleaned up:
-    usedname = usedname ~= "" and usedname or filename or name
-    local basename = keepdir == true and usedname or file.basename(usedname)
-    local basename = gsub(basename,"%./","")
-    local savename = name ~= "" and name or basename
-    if not filetype or filetype == "" then
-        filetype = name and (filename and file.suffix(filename)) or "txt"
-    end
-    savename = file.addsuffix(savename,filetype) -- type is mandate for proper working in viewer
-    local mimetype = specification.mimetype
-    local a = pdfdictionary {
-        Type    = pdfconstant("EmbeddedFile"),
-        Subtype = mimetype and mimetype ~= "" and pdfconstant(mimetype) or nil,
-    }
-    local f
-    if data then
-        f = pdfflushstreamobject(data,a)
-        specification.data = true -- signal that still data but already flushed
-    else
-        local foundname = specification.foundname or filename
-        f = pdfflushstreamfileobject(foundname,a,compress)
-    end
-    local d = pdfdictionary {
-        Type = pdfconstant("Filespec"),
-        F    = pdfstring(savename),
-     -- UF   = pdfstring(savename),
-        UF   = pdfunicode(savename),
-        EF   = pdfdictionary { F = pdfreference(f) },
-        Desc = title ~= "" and pdfunicode(title) or nil,
-     -- AFRelationship = pdfconstant("Source"), -- some day maybe, not mandate
-    }
-    local r = pdfreference(pdfflushobject(d))
-    filestreams[hash] = r
-    return r
-end
-
-function nodeinjections.attachfile(specification)
-    local registered = specification.registered or "<unset>"
-    local data = specification.data
-    local hash
-    local filename
-    if data then
-        hash = md5.HEX(data)
-    else
-        filename = specification.file
-        if not filename or filename == "" then
-            report_attachment("no file specified, using registered %a instead",registered)
-            filename = registered
-            specification.file = registered
+        specification.hash = hash
+        nofattachments = nofattachments + 1
+        local registered = specification.registered or ""
+        local title      = specification.title      or ""
+        local subtitle   = specification.subtitle   or ""
+        local author     = specification.author     or ""
+        local onlyname   = filename and filenameonly(filename) or ""
+        if registered == "" then
+            registered = filename
         end
-        local foundname = resolvers.findbinfile(filename) or ""
-        if foundname == "" or not lfs.isfile(foundname) then
-            report_attachment("invalid filename %a, ignoring registered %a",filename,registered)
-            return nil
+        if author == "" and title ~= "" then
+            author = title
+            title  = onlyname or ""
+        end
+        if author == "" then
+            author = onlyname or "<unknown>"
+        end
+        if title == "" then
+            title = registered
+        end
+        if title == "" and filename then
+            title = onlyname
+        end
+        local aref = attachments[registered]
+        if not aref then
+            aref = codeinjections.embedfile(specification)
+            attachments[registered] = aref
+        end
+        local reference = specification.reference
+        if reference and aref then
+            tobesavedobjrefs[reference] = aref[1]
+        end
+        if not aref then
+            report_attachment("skipping attachment, registered %a",registered)
+            -- already reported
+        elseif specification.method == v_hidden then
+            referenced[hash] = "hidden"
         else
-            specification.foundname = foundname
+            referenced[hash] = "annotation"
+            local name, appearance = analyzesymbol(specification.symbol,attachment_symbols)
+            local flags = specification.flags or 0 -- to keep it expandable
+            local d = pdfdictionary {
+                Subtype  = pdfconstant("FileAttachment"),
+                FS       = aref,
+                Contents = pdfunicode(title),
+                Name     = name,
+                NM       = pdfstring("attachment:"..nofattachments),
+                T        = author ~= "" and pdfunicode(author) or nil,
+                Subj     = subtitle ~= "" and pdfunicode(subtitle) or nil,
+                C        = analyzecolor(specification.colorvalue,specification.colormodel),
+                CA       = analyzetransparency(specification.transparencyvalue),
+                AP       = appearance,
+                OC       = analyzelayer(specification.layer),
+             -- F        = pdfnull(), -- another rediculous need to satisfy validation
+                F        = bit32.band(bit32.bor(flags,4),(1023-1-2-32-256)), -- set 3, clear 1,2,6,9; PDF 32000-1, p385
+            }
+            local width  = specification.width  or 0
+            local height = specification.height or 0
+            local depth  = specification.depth  or 0
+            local box    = hpacknode(nodeinjections.annotation(width,height,depth,d()))
+            box.width    = width
+            box.height   = height
+            box.depth    = depth
+            return box
         end
-        hash = filename
-    end
-    specification.hash = hash
-    nofattachments = nofattachments + 1
-    local registered = specification.registered or ""
-    local title      = specification.title      or ""
-    local subtitle   = specification.subtitle   or ""
-    local author     = specification.author     or ""
-    if registered == "" then
-        registered = filename
-    end
-    if author == "" and title ~= "" then
-        author = title
-        title  = filename or ""
-    end
-    if author == "" then
-        author = filename or "<unknown>"
-    end
-    if title == "" then
-        title = registered
-    end
-    local aref = attachments[registered]
-    if not aref then
-        aref = codeinjections.embedfile(specification)
-        attachments[registered] = aref
-    end
-    local reference = specification.reference
-    if reference and aref then
-        tobesavedobjrefs[reference] = aref[1]
-    end
-    if not aref then
-        report_attachment("skipping attachment, registered %a",registered)
-        -- already reported
-    elseif specification.method == v_hidden then
-        referenced[hash] = "hidden"
-    else
-        referenced[hash] = "annotation"
-        local name, appearance = analyzesymbol(specification.symbol,attachment_symbols)
-        local d = pdfdictionary {
-            Subtype  = pdfconstant("FileAttachment"),
-            FS       = aref,
-            Contents = pdfunicode(title),
-            Name     = name,
-            NM       = pdfstring("attachment:"..nofattachments),
-            T        = author ~= "" and pdfunicode(author) or nil,
-            Subj     = subtitle ~= "" and pdfunicode(subtitle) or nil,
-            C        = analyzecolor(specification.colorvalue,specification.colormodel),
-            CA       = analyzetransparency(specification.transparencyvalue),
-            AP       = appearance,
-            OC       = analyzelayer(specification.layer),
-        }
-        local width  = specification.width  or 0
-        local height = specification.height or 0
-        local depth  = specification.depth  or 0
-        local box    = hpack_node(nodeinjections.annotation(width,height,depth,d()))
-        box.width    = width
-        box.height   = height
-        box.depth    = depth
-        return box
     end
 end
 
@@ -431,6 +473,8 @@ function codeinjections.attachmentid(filename) -- not used in context
     return filestreams[filename]
 end
 
+-- Comments
+
 local nofcomments      = 0
 local usepopupcomments = false
 
@@ -478,11 +522,7 @@ function nodeinjections.comment(specification) -- brrr: seems to be done twice
     local subtitle = specification.subtitle or "" -- as author
     local author   = specification.author   or ""
     local option   = settings_to_hash(specification.option or "")
-    if author == "" then
-        if title == "" then
-            title = tag
-        end
-    else
+    if author ~= "" then
         if subtitle == "" then
             subtitle = title
         elseif title ~= "" then
@@ -490,6 +530,9 @@ function nodeinjections.comment(specification) -- brrr: seems to be done twice
         end
         title = author
     end
+    if title == "" then
+        title = tag
+    end
     local content, richcontent = checkcontent(text,option)
     local d = pdfdictionary {
         Subtype   = pdfconstant("Text"),
@@ -518,12 +561,12 @@ function nodeinjections.comment(specification) -- brrr: seems to be done twice
             Parent  = pdfreference(nd),
         }
         d.Popup = pdfreference(nc)
-        box = hpack_node(
+        box = hpacknode(
             nodeinjections.annotation(0,0,0,d(),nd),
             nodeinjections.annotation(width,height,depth,c(),nc)
         )
     else
-        box = hpack_node(nodeinjections.annotation(width,height,depth,d()))
+        box = hpacknode(nodeinjections.annotation(width,height,depth,d()))
     end
     box.width  = width  -- redundant
     box.height = height -- redundant
@@ -607,23 +650,21 @@ local function insertrenderingwindow(specification)
         Subtype = pdfconstant("Screen"),
         P       = pdfreference(pdfpagereference(page)),
         A       = a, -- needed in order to make the annotation clickable (i.e. don't bark)
+        T       = pdfunicode(label), -- for JS
         Border  = bs,
         C       = bc,
         AA      = actions,
     }
     local width = specification.width or 0
     local height = specification.height or 0
-    if height == 0 or width == 0 then
-        -- todo: sound needs no window
-    end
-    write_node(nodeinjections.annotation(width,height,0,d(),r)) -- save ref
+    context(nodeinjections.annotation(width,height,0,d(),r)) -- save ref
     return pdfreference(r)
 end
 
 -- some dictionaries can have a MH (must honor) or BE (best effort) capsule
 
 local function insertrendering(specification)
-    local label = specification.label
+    local label  = specification.label
     local option = settings_to_hash(specification.option)
     if not mf[label] then
         local filename = specification.filename
@@ -653,21 +694,23 @@ local function insertrendering(specification)
      --          B = start,
      --     }
      -- }
-     -- local parameters = pdfdictionary {
-     --     Type = pdfconstant(MediaPermissions),
-     --     TF   = pdfstring("TEMPALWAYS") }, -- TEMPNEVER TEMPEXTRACT TEMPACCESS TEMPALWAYS
-     -- }
+        local parameters = pdfdictionary {
+            Type = pdfconstant("MediaPermissions"),
+            TF   = pdfstring("TEMPALWAYS"), -- TEMPNEVER TEMPEXTRACT TEMPACCESS TEMPALWAYS / needed for acrobat/wmp
+        }
         local descriptor = pdfdictionary {
             Type = pdfconstant("Filespec"),
             F    = filename,
         }
         if isurl then
             descriptor.FS = pdfconstant("URL")
+            descriptor = pdfreference(pdfflushobject(descriptor))
         elseif option[v_embed] then
-            descriptor.EF = codeinjections.embedfile {
-                file     = filename,
-                mimetype = mimetype, -- yes or no
-                compress = false,
+            descriptor = codeinjections.embedfile {
+                file           = filename,
+                mimetype       = mimetype, -- yes or no
+                compress       = option[v_compress] or false,
+                forcereference = true,
             }
         end
         local clip = pdfdictionary {
@@ -676,13 +719,13 @@ local function insertrendering(specification)
             N    = label,
             CT   = mimetype,
             Alt  = pdfarray { "", "file not found" }, -- language id + message
-            D    = pdfreference(pdfflushobject(descriptor)),
-         -- P    = pdfreference(pdfflushobject(parameters)),
+            D    = descriptor,
+            P    = pdfreference(pdfflushobject(parameters)),
         }
         local rendition = pdfdictionary {
             Type = pdfconstant("Rendition"),
             S    = pdfconstant("MR"),
-            N    = label,
+            N    = pdfunicode(label),
             C    = pdfreference(pdfflushobject(clip)),
         }
         mf[label] = pdfreference(pdfflushobject(rendition))
@@ -720,6 +763,21 @@ function codeinjections.processrendering(label)
     end
 end
 
+-- needed mapping for access from JS
+
+local function flushrenderings()
+    if next(mf) then
+        local r = pdfarray()
+        for label, reference in sortedhash(mf) do
+            r[#r+1] = pdfunicode(label)
+            r[#r+1] = reference -- already a reference
+        end
+        lpdf.addtonames("Renditions",pdfreference(pdfflushobject(pdfdictionary{ Names = r })))
+    end
+end
+
+lpdf.registerdocumentfinalizer(flushrenderings,"renderings")
+
 function codeinjections.insertrenderingwindow(specification)
     local label = specification.label
     codeinjections.processrendering(label)
