@@ -6,28 +6,12 @@ if not modules then modules = { } end modules ['mlib-run'] = {
     license   = "see context related readme files",
 }
 
--- cmyk       -> done, native
--- spot       -> done, but needs reworking (simpler)
--- multitone  ->
--- shade      -> partly done, todo: cm
--- figure     -> done
--- hyperlink  -> low priority, easy
-
--- new * run
--- or
--- new * execute^1 * finish
-
--- a*[b,c] == b + a * (c-b)
-
---[[ldx--
-<p>The directional helpers and pen analysis are more or less translated from the
-<l n='c'/> code. It really helps that Taco know that source so well. Taco and I spent
-quite some time on speeding up the <l n='lua'/> and <l n='c'/> code. There is not
-much to gain, especially if one keeps in mind that when integrated in <l n='tex'/>
-only a part of the time is spent in <l n='metapost'/>. Of course an integrated
-approach is way faster than an external <l n='metapost'/> and processing time
-nears zero.</p>
---ldx]]--
+-- The directional helpers and pen analysis are more or less translated from the C
+-- code. It really helps that Taco know that source so well. Taco and I spent quite
+-- some time on speeding up the Lua and C code. There is not much to gain,
+-- especially if one keeps in mind that when integrated in TeX only a part of the
+-- time is spent in MetaPost. Of course an integrated approach is way faster than an
+-- external MetaPost and processing time nears zero.
 
 local type, tostring, tonumber, next = type, tostring, tonumber, next
 local find, striplines = string.find, utilities.strings.striplines
@@ -219,9 +203,6 @@ function metapost.checkformat(mpsinput,method)
  -- if foundfile == "" then
  --     foundfile  = find_file(file.replacesuffix(mpsinput,"mpvi")) or ""
  -- end
-    if CONTEXTLMTXMODE > 0 and foundfile == "" then
-        foundfile  = find_file(file.replacesuffix(mpsinput,"mpxl")) or ""
-    end
     if foundfile == "" then
         foundfile  = find_file(file.replacesuffix(mpsinput,"mpiv")) or ""
     end
