@@ -11,18 +11,25 @@ if not modules then modules = { } end modules ['mult-low'] = {
 return {
     ["constants"] = {
         --
-        "zerocount", "minusone", "minustwo", "plusone", "plustwo", "plusthree", "plusfour", "plusfive",
-        "plussix", "plusseven", "pluseight", "plusnine", "plusten", "pluseleven", "plustwelve", "plussixteen",
-        "plusfifty", "plushundred", "plusonehundred", "plustwohundred", "plusfivehundred",
+        "zerocount", "minusone", "minustwo",
+        "plusone", "plustwo", "plusthree", "plusfour", "plusfive", "plussix", "plusseven", "pluseight",
+        "plusnine", "plusten", "pluseleven", "plustwelve", "plusthirteen", "plusfourteen", "plusfifteen", "plussixteen",
+        "plusfifty", "plussixty", "plushundred", "plusonehundred", "plustwohundred", "plusfivehundred",
+        "plusonehundredtwentyfive", "plustwohundredfifty",
         "plusthousand", "plustenthousand", "plustwentythousand", "medcard", "maxcard", "maxcardminusone", "maxiterator",
+        "plusonethousand", "plustwothousand", "plusthreethousand", "plusfourthousand", "plusfivethousand",
+        "plussixthousand", "plusseventhousand", "pluseightthousand", "plusninethousand",
+        "plusfifteenhundred", "plustwentyfivehundred", "plusfortyfivehundred", "plusseventyfivehundred",
+        "plusninetynine", "plusthirtytwo",  "plusfourtytwo", "plustwentyfour", "plustwentyfive",
+        "plusthousandfifty", "plustwelvehundredfifty",
         "zeropoint", "onepoint", "halfapoint", "onebasepoint", "maxcount", "maxdimen", "scaledpoint", "thousandpoint",
-        "points", "halfpoint",
+        "points", "halfpoint", "twopoints", "threepoints",
         "zeroskip", "centeringskip", "stretchingskip", "shrinkingskip",
         "centeringfillskip", "stretchingfillskip", "shrinkingfillskip",
         "centeringfilllskip", "stretchingfilllskip", "shrinkingfilllskip",
         "zeromuskip", "onemuskip",
         "pluscxxvii", "pluscxxviii", "pluscclv", "pluscclvi",
-        "normalpagebox",
+        "normalpagebox", "delayedpagebox",
         "binaryshiftedten", "binaryshiftedtwenty", "binaryshiftedthirty",
         "thickermuskip", "continuationmuskip", "fencemuskip", "mathinterwordmuskip",
         "zerofloat",
@@ -33,6 +40,8 @@ return {
         --
         "emptytoks", "empty", "undefined",
         --
+        "blankspace", "relaxedspace", -- old new
+        --
         "prerollrun",
         --
         "voidbox", "emptybox", "emptyvbox", "emptyhbox",
@@ -59,7 +68,7 @@ return {
         "inicatcodes",
         "ctxcatcodes", "texcatcodes", "notcatcodes", "txtcatcodes", "vrbcatcodes",
         "prtcatcodes", "nilcatcodes", "luacatcodes", "tpacatcodes", "tpbcatcodes",
-        "xmlcatcodes", "ctdcatcodes", "rlncatcodes",
+        "xmlcatcodes", "ctdcatcodes", "rlncatcodes", "hshcatcodes",
         --
         "escapecatcode", "begingroupcatcode", "endgroupcatcode", "mathshiftcatcode", "alignmentcatcode",
         "endoflinecatcode", "parametercatcode", "superscriptcatcode", "subscriptcatcode", "ignorecatcode",
@@ -82,8 +91,10 @@ return {
         "loosenessfrozenparcode", "lastlinefrozenparcode", "linepenaltyfrozenparcode", "clubpenaltyfrozenparcode",
         "widowpenaltyfrozenparcode", "displaypenaltyfrozenparcode", "brokenpenaltyfrozenparcode",
         "demeritsfrozenparcode", "shapefrozenparcode", "linefrozenparcode", "hyphenationfrozenparcode",
-        "shapingpenaltyfrozenparcode", "orphanpenaltyfrozenparcode", "allfrozenparcode",
+        "shapingpenaltyfrozenparcode", "orphanpenaltyfrozenparcode", "toddlerpenaltyfrozenparcode",
         "emergencyfrozenparcode", "parpassesfrozenparcode", "singlelinepenaltyfrozenparcode",
+        "hyphenpenaltyfrozenparcode", "exhyphenpenaltyfrozenparcode", "linebreakchecksfrozenparcode",
+        "twindemeritsfrozenparcode", "fitnessclassesfrozenparcode", "allfrozenparcode",
         --
         "activemathcharcode",
         --
@@ -139,6 +150,8 @@ return {
         "halfwaytotallistanchorcode", "halfwayheightlistanchorcode", "halfwaydepthlistanchorcode",
         "halfwayleftlistanchorcode", "halfwayrightlistanchorcode",
         --
+        "underfullbadnessmodecode", "loosebadnessmodecode", "tightbadnessmodecode", "overfullbadnessmodecode",
+        --
         "negatexlistsigncode", "negateylistsigncode", "negatelistsigncode",
         --
         "fontslantperpoint", "fontinterwordspace", "fontinterwordstretch", "fontinterwordshrink",
@@ -164,9 +177,13 @@ return {
         "doifelsedocumentvariable", "doifdocumentvariableelse", "doifdocumentvariable", "doifnotdocumentvariable",
         "startmodule", "stopmodule", "usemodule", "usetexmodule", "useluamodule","setupmodule","currentmoduleparameter","moduleparameter",
         "everystarttext", "everystoptext", "everyforgetall", "luaenvironment",
+        --
+        -- -- maybe do all the important every's
+        --
+     -- "everymathematics", "everyinsidemathematics",
         --
         "startTEXpage", "stopTEXpage",
-    --  "startMPpage", "stopMPpage", -- already catched by nested lexer
+     -- "startMPpage", "stopMPpage", -- already catched by nested lexer
         --
         "enablemode", "disablemode", "preventmode", "definemode",
         "globalenablemode", "globaldisablemode", "globalpreventmode",
@@ -188,8 +205,13 @@ return {
         "mathmaybeordinarycode", "mathmayberelationcode", "mathmaybebinarycode", "mathnumbergroupcode",
         "mathchemicalbondcode", "mathimplicationcode",
         --
+        "mathnormalstylepreset", "mathcrampedstylepreset", "mathsubscriptstylepreset", "mathsuperscriptstylepreset", "mathsmallstylepreset",
+        "mathsmallerstylepreset", "mathnumeratorstylepreset", "mathdenominatorstylepreset", "mathdoublesuperscriptstylepreset",
+        --
         "constantnumber", "constantnumberargument", "constantdimen", "constantdimenargument", "constantemptyargument",
         --
+        "periodicshrink",
+        --
         "luastringsep", "!!bs", "!!es",
         --
         "lefttorightmark", "righttoleftmark", "lrm", "rlm",
@@ -204,6 +226,8 @@ return {
         --
         "Ux", "eUx",
         --
+        "mathstylevariantidentity", "mathstylevariantcramped", "mathstylevariantuncramped",
+        --
      -- "parfillleftskip", "parfillrightskip",
         --
         "startlmtxmode", "stoplmtxmode", "startmkivmode", "stopmkivmode",
@@ -215,12 +239,14 @@ return {
         "automaticpenaltyhyphenationcode", "explicitpenaltyhyphenationcode", "permitgluehyphenationcode", "permitallhyphenationcode",
         "permitmathreplacehyphenationcode", "forcecheckhyphenationcode", "lazyligatureshyphenationcode", "forcehandlerhyphenationcode",
         "feedbackcompoundhyphenationcode", "ignoreboundshyphenationcode", "partialhyphenationcode", "completehyphenationcode",
+        "replaceapostrophehyphenationcode",
         --
         "normalizelinenormalizecode", "parindentskipnormalizecode", "swaphangindentnormalizecode", "swapparsshapenormalizecode",
         "breakafterdirnormalizecode", "removemarginkernsnormalizecode", "clipwidthnormalizecode", "flattendiscretionariesnormalizecode",
-        "discardzerotabskipsnormalizecode", "flattenhleadersnormalizecode",
+        "discardzerotabskipsnormalizecode", "flattenhleadersnormalizecode", "balanceinlinemathnormalizecode",
         --
         "normalizeparnormalizeparcode", "flattenvleadersnormalizeparcode", "limitprevgrafnormalizeparcode",
+        "keepinterlinepenaltiesnormalizeparcode", "removetrailingspacesnormalizeparcode",
         --
         "nopreslackclassoptioncode", "nopostslackclassoptioncode",
         "lefttopkernclassoptioncode", "righttopkernclassoptioncode", "leftbottomkernclassoptioncode", "rightbottomkernclassoptioncode",
@@ -238,17 +264,29 @@ return {
         "noleftkernglyphoptioncode",  "norightkernglyphoptioncode",
         "noleftligatureglyphoptioncode", "norightligatureglyphoptioncode",
         "textcheckitalicglyphoptioncode", "mathcheckitalicglyphoptioncode",
+        "weightlessglyphoptioncode", "spacefactoroverloadglyphoptioncode",
+        "checktoddlerglyphoptioncode", "checktwinglyphoptioncode",
         --
-     -- "noitaliccorrectionmathoptioncode", "noleftpairkernmathoptioncode", "norightpairkernmathoptioncode",
-     -- "autodiscretionarymathoptioncode", "fulldiscretionarymathoptioncode",
-     -- "ignoredcharactermathoptioncode", "islargeoperatormathoptioncode", "hasitalicshapemathoptioncode",
+        "ignoretwincharactercontrolcode",
+        --
+        "repeatspecificationoptioncode", "doublespecificationoptioncode", "largestspecificationoptioncode",
+        "presetsspecificationoptioncode", "integerspecificationoptioncode", "finalspecificationoptioncode",
+        "defaultspecificationoptioncode", "ignorespecificationoptioncode", "rotatespecificationoptioncode",
+        --
+        "shortmathoptioncode", "orphanedmathoptioncode", "displaymathoptioncode", "crampedmathoptioncode",
+        "nosnappingmathoptioncode",
+        --
+     -- "noitaliccorrectionkerneloptioncode", "noleftpairkernkerneloptioncode", "norightpairkernkerneloptioncode",
+     -- "autodiscretionarykerneloptioncode", "fulldiscretionarykerneloptioncode",
+     -- "ignoredcharacterkerneloptioncode", "islargeoperatorkerneloptioncode", "hasitalicshapekerneloptioncode",
         --
         "nokerningcode", "noligaturingcode", "noitalicscode",
         --
-        "indecentparpassclasses", "looseparpassclasses", "tightparpassclasses",
-        "verylooseparpassclass", "looseparpassclass", "semilooseparpassclass",
-        "decentparpassclass", "almostdecentparpassclasses",
-        "semitightparpassclass", "tightparpassclass",
+        "allparpassclasses", "indecentparpassclasses", "looseparpassclasses", "tightparpassclasses", "almostdecentparpassclasses",
+        --
+        "verylooseparpassclass", "looseparpassclass", "almostlooseparpassclass", "barelylooseparpassclass",
+        "decentparpassclass",
+        "verytightparpassclass", "tightparpassclass", "almosttightparpassclass", "barelytightparpassclass",
         --
         "frozenflagcode", "tolerantflagcode", "protectedflagcode", "primitiveflagcode", "permanentflagcode", "noalignedflagcode", "immutableflagcode",
         "mutableflagcode", "globalflagcode", "overloadedflagcode", "immediateflagcode", "conditionalflagcode", "valueflagcode", "instanceflagcode",
@@ -260,9 +298,18 @@ return {
         "noitaliccorrectiondiscoptioncode", "nozeroitaliccorrectiondiscoptioncode",
         "textcheckitalicdiscoptioncode",
         --
+        "ignoreprevdepthmvloptioncode", "noprevdepthmvloptioncode", "discardtopmvloptioncode", "discardbottommvloptioncode",
+        --
         "continueifinputfile", "continuewhenlmtxmode", "continuewhenmkivmode",
         --
-        "uunit"
+        "uunit",
+        --
+        "defaultdisplaywidowpenalty", "defaultwidowpenalty", "defaultclubpenalty", "defaultbrokenpenalty",
+        "defaultgriddisplaywidowpenalty", "defaultgridwidowpenalty", "defaultgridclubpenalty", "defaultgridbrokenpenalty",
+        --
+        "luametatexverboseversion", "luametatexfunctionality",
+        --
+        "currentverbosemathstyle",
     },
     ["helpers"] = {
         --
@@ -288,7 +335,7 @@ return {
         "globalsetmode", "globalresetmode", "globalsetsystemmode", "globalresetsystemmode",
         "booleanmodevalue",
         --
-        "newcount", "newdimen", "newskip", "newmuskip", "newbox", "newtoks", "newread", "newwrite", "newmarks", "newinsert", "newattribute", "newif", "newfloat",
+        "newcount", "newdimen", "newskip", "newmuskip", "newbox", "newtoks", "newread", "newwrite", "newmarks", "newinsert", "newattribute", "newif", "newfloat", "newmvl",
         "newlanguage", "newfamily", "newfam", "newhelp", -- not used
         "newuserunit",
         --
@@ -364,6 +411,8 @@ return {
         "scratchbox", "globalscratchbox", "privatescratchbox",
         "scratchmacro", "scratchmacroone", "scratchmacrotwo",
         --
+        "ignoredtoks",
+        --
         "scratchconditiontrue", "scratchconditionfalse", "ifscratchcondition",
         "scratchconditiononetrue", "scratchconditiononefalse", "ifscratchconditionone",
         "scratchconditiontwotrue", "scratchconditiontwofalse", "ifscratchconditiontwo",
@@ -400,7 +449,7 @@ return {
         "scratchmuskipone", "scratchmuskiptwo", "scratchmuskipthree", "scratchmuskipfour", "scratchmuskipfive", "scratchmuskipsix",
         "scratchtoksone", "scratchtokstwo", "scratchtoksthree", "scratchtoksfour", "scratchtoksfive", "scratchtokssix",
         "scratchboxone", "scratchboxtwo", "scratchboxthree", "scratchboxfour", "scratchboxfive", "scratchboxsix",
-        "scratchnx", "scratchny", "scratchmx", "scratchmy",
+        "scratchnx", "scratchny", "scratchmx", "scratchmy", "scratchsx", "scratchsy",
         "scratchunicode", "scratchunicodeone", "scratchunicodetwo", "scratchunicodethree",
         "scratchmin", "scratchmax",
         "scratchread", "scratchwrite",
@@ -451,7 +500,7 @@ return {
         --
         -- "everyendpar",
         --
-        "endgraf", "endpar", "reseteverypar", "finishpar", "empty", "null", "space", "quad", "enspace", "emspace", "charspace", "nbsp", "crlf",
+        "endgraf", "endpar", "reseteverypar", "finishpar", "null", "space", "quad", "enspace", "emspace", "charspace", "nbsp", "crlf",
         "obeyspaces", "obeylines", "obeytabs", "obeypages", "obeyedspace", "obeyedline", "obeyedtab", "obeyedpage",
         "normalspace", "naturalspace", "controlspace", "normalspaces",
         "ignoretabs", "ignorelines", "ignorepages", "ignoreeofs", "setcontrolspaces",
@@ -484,12 +533,14 @@ return {
         --
         "getdummyparameters", "dummyparameter", "directdummyparameter", "setdummyparameter", "letdummyparameter",
         "setexpandeddummyparameter", "resetdummyparameter",
-        "usedummystyleandcolor", "usedummystyleparameter", "usedummycolorparameter",
+        "usedummystyleandcolor", "usedummystyleparameter", "usedummycolorparameter", "usedummylanguageparameter",
         --
         "processcommalist", "processcommacommand", "quitcommalist", "quitprevcommalist",
         "processaction", "processallactions", "processfirstactioninset", "processallactionsinset",
         --
-        "unexpanded", "expanded", "startexpanded", "stopexpanded", "protect", "unprotect",
+        "unexpanded",
+     -- "expanded",
+        "startexpanded", "stopexpanded", "protect", "unprotect",
         --
         "firstofoneargument",
         "firstoftwoarguments", "secondoftwoarguments",
@@ -530,18 +581,22 @@ return {
         --
         "twodigits","threedigits",
         --
+        "jobposx", "jobposy", "jobposw", "jobposh", "jobposd",
+        --
         "leftorright",
         --
         "offinterlineskip", "oninterlineskip", "nointerlineskip",
         --
         "strut", "halfstrut", "quarterstrut", "depthstrut", "halflinestrut", "noheightstrut", "setstrut", "strutbox", "strutht", "strutdp", "strutwd", "struthtdp", "strutgap", "begstrut", "endstrut", "lineheight",
-        "leftboundary", "rightboundary", "signalcharacter",
+        "leftboundary", "rightboundary", "signalcharacter", "signalglyph",
         "ascender", "descender", "capheight",
         --
         "aligncontentleft", "aligncontentmiddle", "aligncontentright",
         --
         "shiftbox", "vpackbox", "hpackbox", "vpackedbox", "hpackedbox",
         --
+        "vreflected",
+        --
      -- "ordordspacing", "ordopspacing", "ordbinspacing", "ordrelspacing",
      -- "ordopenspacing", "ordclosespacing", "ordpunctspacing", "ordinnerspacing",
      -- "ordfracspacing", "ordradspacing", "ordmiddlespacing", "ordaccentspacing",
@@ -634,8 +689,6 @@ return {
         "updateparagraphproperties", "updateparagraphpenalties", "updateparagraphdemerits", "updateparagraphshapes", "updateparagraphlines",
         "updateparagraphpasses",
         --
-        "lastlinewidth",
-        --
         "assumelongusagecs",
         --
         "righttolefthbox", "lefttorighthbox", "righttoleftvbox", "lefttorightvbox", "righttoleftvtop", "lefttorightvtop",
@@ -643,12 +696,14 @@ return {
         "autodirhbox", "autodirvbox", "autodirvtop",
         "leftorrighthbox", "leftorrightvbox", "leftorrightvtop",
         "lefttoright", "righttoleft", "checkedlefttoright", "checkedrighttoleft",
+        "foolbidimode",
         "synchronizelayoutdirection","synchronizedisplaydirection","synchronizeinlinedirection",
         "dirlre", "dirrle", "dirlro", "dirrlo",
         "rtltext", "ltrtext",
         --
         "lesshyphens", "morehyphens", "nohyphens", "dohyphens", "dohyphencollapsing", "nohyphencollapsing",
         "compounddiscretionary",
+        "doapostrophes", "noapostrophes",
         --
         "Ucheckedstartdisplaymath", "Ucheckedstopdisplaymath",
         --
@@ -677,6 +732,7 @@ return {
         "boxlinels", "boxliners", "boxlinelh", "boxlinerh",
         "boxlinelp", "boxlinerp", "boxlinein",
         "boxrangewd", "boxrangeht", "boxrangedp",
+        "boxlinemaxwd","boxlinemaxht", "boxlinemaxdp",
         --
         "bitwiseset", "bitwiseand", "bitwiseor", "bitwisexor", "bitwisenot", "bitwisenil",
         "ifbitwiseand", "bitwise", "bitwiseshift", "bitwiseflip",
@@ -687,6 +743,7 @@ return {
         --
         "noligaturing", "nokerning", "noexpansion", "noprotrusion",
         "noleftkerning", "noleftligaturing", "norightkerning", "norightligaturing", "noitaliccorrection",
+        "overloadspacefactor",
          --
         "futureletnexttoken", "defbackslashbreak", "letbackslashbreak",
         --
@@ -701,7 +758,7 @@ return {
         "parcontextcodestring",
         --
         "newlocalcount", "newlocaldimen", "newlocalskip", "newlocalmuskip", "newlocaltoks", "newlocalbox",
-        "newlocalwrite", "newlocalread",
+        "newlocalwrite", "newlocalread", "newlocalfloat",
         "setnewlocalcount", "setnewlocaldimen", "setnewlocalskip", "setnewlocalmuskip", "setnewlocaltoks", "setnewlocalbox",
         --
         "ifexpression",
@@ -730,5 +787,26 @@ return {
         "ignorefile",
         --
         "boxwidth", "boxheight", "boxdepth",
+        --
+        "shiftparshape", "rotateparshape",
+        --
+        "granularfitnessclasses", "granularadjacentdemerits",
+        "matchallfitnessclasses",
+        --
+        "defaultmathforwardpenalties", "defaultmathbackwardpenalties",
+        "optimalmathforwardpenalties", "optimalmathbackwardpenalties",
+        "lesswidowpenalties", "lessclubpenalties", "lessbrokenpenalties",
+        "strictwidowpenalties", "strictwidowpenaltiestwo", "strictwidowpenaltiesthree", "strictwidowpenaltiesfour",
+        "strictclubpenalties", "strictclubpenaltiestwo", "strictclubpenaltiesthree", "strictclubpenaltiesfour",
+        "strictbrokenpenalties",
+        "lessorphanpenalties", "lessorphanpenaltiestwo", "lessorphanpenaltiesthree","lessorphanpenaltiesfour",
+        --
+        "nohpenalties", "novpenalties",
+        --
+        "toddlerpenalty", "orphanpenalty",
+        --
+        "digitspace",
+        --
+        "inhibitprimitive"
     }
 }
