@@ -42,7 +42,6 @@ return {
   "parshapedimen",
   "parshapeindent",
   "parshapelength",
-  "predisplaydirection",
   "protected",
   "savinghyphcodes",
   "savingvdiscards",
@@ -67,7 +66,6 @@ return {
   "Uabovewithdelims",
   "Uatop",
   "Uatopwithdelims",
-  "Uchar",
   "Udelcode",
   "Udelimited",
   "Udelimiter",
@@ -91,11 +89,7 @@ return {
   "Umathaxis",
   "Umathbottomaccentvariant",
   "Umathchar",
-  "Umathcharclass",
   "Umathchardef",
-  "Umathcharfam",
-  "Umathcharslot",
-  "Umathclass",
   "Umathcode",
   "Umathconnectoroverlapmin",
   "Umathdegreevariant",
@@ -105,9 +99,8 @@ return {
   "Umathdelimitershortfall",
   "Umathdelimiterundervariant",
   "Umathdenominatorvariant",
-  "Umathdict",
   "Umathdictdef",
-  "Umathdiscretionary",
+  "Umathexheight",
   "Umathextrasubpreshift",
   "Umathextrasubprespace",
   "Umathextrasubshift",
@@ -152,15 +145,12 @@ return {
   "Umathoverlayaccentvariant",
   "Umathoverlinevariant",
   "Umathphantom",
-  "Umathpresubshiftdistance",
-  "Umathpresupshiftdistance",
   "Umathprimeraise",
   "Umathprimeraisecomposed",
   "Umathprimeshiftdrop",
   "Umathprimeshiftup",
   "Umathprimespaceafter",
   "Umathprimevariant",
-  "Umathprimewidth",
   "Umathquad",
   "Umathradicaldegreeafter",
   "Umathradicaldegreebefore",
@@ -179,20 +169,21 @@ return {
   "Umathsource",
   "Umathspaceafterscript",
   "Umathspacebeforescript",
+  "Umathspacebetweenscript",
   "Umathstackdenomdown",
   "Umathstacknumup",
   "Umathstackvariant",
   "Umathstackvgap",
+  "Umathsubscriptsnap",
   "Umathsubscriptvariant",
-  "Umathsubshiftdistance",
   "Umathsubshiftdown",
   "Umathsubshiftdrop",
   "Umathsubsupshiftdown",
   "Umathsubsupvgap",
   "Umathsubtopmax",
   "Umathsupbottommin",
+  "Umathsuperscriptsnap",
   "Umathsuperscriptvariant",
-  "Umathsupshiftdistance",
   "Umathsupshiftdrop",
   "Umathsupshiftup",
   "Umathsupsubbottommax",
@@ -210,26 +201,16 @@ return {
   "Umathxscale",
   "Umathyscale",
   "Umiddle",
-  "Unosubprescript",
-  "Unosubscript",
-  "Unosuperprescript",
-  "Unosuperscript",
   "Uoperator",
   "Uover",
   "Uoverdelimiter",
   "Uoverwithdelims",
-  "Uprimescript",
   "Uradical",
   "Uright",
   "Uroot",
   "Urooted",
-  "Ushiftedsubprescript",
-  "Ushiftedsubscript",
-  "Ushiftedsuperprescript",
-  "Ushiftedsuperscript",
   "Uskewed",
   "Uskewedwithdelims",
-  "Ustack",
   "Ustartdisplaymath",
   "Ustartmath",
   "Ustartmathmode",
@@ -238,17 +219,15 @@ return {
   "Ustopmathmode",
   "Ustretched",
   "Ustretchedwithdelims",
-  "Ustyle",
-  "Usubprescript",
-  "Usubscript",
-  "Usuperprescript",
-  "Usuperscript",
   "Uunderdelimiter",
   "Uvextensible",
+  "additionalpageskip",
+  "adjacentdemerits",
   "adjustspacing",
   "adjustspacingshrink",
   "adjustspacingstep",
   "adjustspacingstretch",
+  "advanceby",
   "afterassigned",
   "aftergrouped",
   "aliased",
@@ -268,30 +247,64 @@ return {
   "alluncrampedstyles",
   "allunsplitstyles",
   "amcode",
+  "associateunit",
+  "atendoffile",
+  "atendoffiled",
   "atendofgroup",
   "atendofgrouped",
   "attribute",
+  "attributeboundary",
   "attributedef",
   "automaticdiscretionary",
   "automatichyphenpenalty",
   "automigrationmode",
   "autoparagraphmode",
+  "balanceadjdemerits",
+  "balancebottomskip",
+  "balanceboundary",
+  "balancebreakpasses",
+  "balancechecks",
+  "balanceemergencyshrink",
+  "balanceemergencystretch",
+  "balancefinalpenalties",
+  "balancelineheight",
+  "balancelooseness",
+  "balancepasses",
+  "balancepenalty",
+  "balanceshape",
+  "balanceshapebottomspace",
+  "balanceshapetopspace",
+  "balanceshapevsize",
+  "balancetolerance",
+  "balancetopskip",
+  "balancevsize",
   "begincsname",
   "beginlocalcontrol",
   "beginmathgroup",
+  "beginmvl",
   "beginsimplegroup",
+  "bottomskip",
   "boundary",
   "boxadapt",
   "boxanchor",
   "boxanchors",
   "boxattribute",
   "boxdirection",
+  "boxfinalize",
   "boxfreeze",
   "boxgeometry",
+  "boxinserts",
+  "boxlimit",
+  "boxlimitate",
+  "boxlimitmode",
+  "boxmigrate",
   "boxorientation",
   "boxrepack",
   "boxshift",
+  "boxshrink",
   "boxsource",
+  "boxstretch",
+  "boxsubtype",
   "boxtarget",
   "boxtotal",
   "boxvadjust",
@@ -299,46 +312,69 @@ return {
   "boxxoffset",
   "boxymove",
   "boxyoffset",
+  "brokenpenalties",
   "catcodetable",
+  "cccode",
   "cdef",
   "cdefcsname",
   "cfcode",
   "clearmarks",
   "constant",
+  "constrained",
   "copymathatomrule",
   "copymathparent",
   "copymathspacing",
+  "copysplitdiscards",
   "crampeddisplaystyle",
   "crampedscriptscriptstyle",
   "crampedscriptstyle",
   "crampedtextstyle",
   "csactive",
+  "csnamestring",
   "csstring",
   "currentloopiterator",
   "currentloopnesting",
+  "currentlysetmathstyle",
   "currentmarks",
+  "currentstacksize",
   "dbox",
   "defcsname",
   "deferred",
+  "detokened",
   "detokenized",
   "dimensiondef",
+  "dimexperimental",
   "dimexpression",
   "directlua",
+  "discretionaryoptions",
+  "divideby",
+  "doublepenaltymode",
   "dpack",
   "dsplit",
   "edefcsname",
+  "edivide",
+  "edivideby",
   "efcode",
+  "emergencyleftskip",
+  "emergencyrightskip",
+  "emptyparagraphmode",
   "endlocalcontrol",
   "endmathgroup",
+  "endmvl",
   "endsimplegroup",
   "enforced",
+  "eofinput",
+  "etexexprmode",
   "etoks",
   "etoksapp",
   "etokspre",
   "eufactor",
   "everybeforepar",
   "everymathatom",
+  "everyparbegin",
+  "everyparend",
   "everytab",
+  "exapostrophechar",
   "exceptionpenalty",
   "expand",
   "expandactive",
@@ -347,27 +383,41 @@ return {
   "expandcstoken",
   "expanded",
   "expandedafter",
+  "expandeddetokenize",
+  "expandedendless",
   "expandedloop",
+  "expandedrepeat",
+  "expandparameter",
   "expandtoken",
+  "expandtoks",
   "explicitdiscretionary",
   "explicithyphenpenalty",
+  "explicititaliccorrection",
+  "explicitspace",
   "firstvalidlanguage",
+  "fitnessclasses",
   "float",
   "floatdef",
   "floatexpr",
   "flushmarks",
+  "flushmvl",
   "fontcharba",
   "fontcharta",
   "fontid",
+  "fontidentifier",
   "fontmathcontrol",
   "fontspecdef",
   "fontspecid",
   "fontspecifiedname",
   "fontspecifiedsize",
   "fontspecscale",
+  "fontspecslant",
+  "fontspecweight",
   "fontspecxscale",
   "fontspecyscale",
   "fonttextcontrol",
+  "forcedleftcorrection",
+  "forcedrightcorrection",
   "formatname",
   "frozen",
   "futurecsname",
@@ -376,6 +426,7 @@ return {
   "futureexpandis",
   "futureexpandisap",
   "gdefcsname",
+  "givenmathstyle",
   "gleaders",
   "glet",
   "gletcsname",
@@ -388,8 +439,10 @@ return {
   "glyphscriptfield",
   "glyphscriptscale",
   "glyphscriptscriptscale",
+  "glyphslant",
   "glyphstatefield",
   "glyphtextscale",
+  "glyphweight",
   "glyphxoffset",
   "glyphxscale",
   "glyphxscaled",
@@ -398,11 +451,13 @@ return {
   "glyphyscaled",
   "gtoksapp",
   "gtokspre",
+  "hbadnessmode",
   "hccode",
   "hjcode",
   "hmcode",
   "holdingmigrations",
   "hpack",
+  "hpenalty",
   "hyphenationmin",
   "hyphenationmode",
   "ifabsdim",
@@ -412,11 +467,14 @@ return {
   "ifboolean",
   "ifchkdim",
   "ifchkdimension",
+  "ifchkdimexpr",
   "ifchknum",
   "ifchknumber",
+  "ifchknumexpr",
   "ifcmpdim",
   "ifcmpnum",
   "ifcondition",
+  "ifcramped",
   "ifcstok",
   "ifdimexpression",
   "ifdimval",
@@ -427,8 +485,14 @@ return {
   "ifhastok",
   "ifhastoks",
   "ifhasxtoks",
+  "ifinalignment",
   "ifincsname",
   "ifinsert",
+  "ifintervaldim",
+  "ifintervalfloat",
+  "ifintervalnum",
+  "iflastnamedcs",
+  "iflist",
   "ifmathparameter",
   "ifmathstyle",
   "ifnumexpression",
@@ -441,28 +505,43 @@ return {
   "ifzerofloat",
   "ifzeronum",
   "ignorearguments",
-  "ignoredepthcriterium",
+  "ignoredepthcriterion",
+  "ignorenestedupto",
   "ignorepars",
+  "ignorerest",
+  "ignoretokens",
+  "ignoreupto",
   "immediate",
   "immutable",
+  "indexedsubprescript",
+  "indexedsubscript",
+  "indexedsuperprescript",
+  "indexedsuperscript",
   "indexofcharacter",
   "indexofregister",
   "inherited",
   "initcatcodetable",
+  "initialpageskip",
+  "initialtopskip",
   "insertbox",
   "insertcopy",
   "insertdepth",
+  "insertdirection",
   "insertdistance",
   "insertheight",
   "insertheights",
   "insertlimit",
+  "insertlinedepth",
+  "insertlineheight",
   "insertmaxdepth",
   "insertmode",
   "insertmultiplier",
   "insertpenalty",
   "insertprogress",
+  "insertshrink",
   "insertstorage",
   "insertstoring",
+  "insertstretch",
   "insertunbox",
   "insertuncopy",
   "insertwidth",
@@ -471,16 +550,20 @@ return {
   "lastarguments",
   "lastatomclass",
   "lastboundary",
-  "lastchkdim",
-  "lastchknum",
+  "lastchkdimension",
+  "lastchknumber",
   "lastleftclass",
+  "lastlinecount",
+  "lastlinewidth",
   "lastloopiterator",
   "lastnamedcs",
   "lastnodesubtype",
   "lastpageextra",
   "lastparcontext",
+  "lastpartrigger",
   "lastrightclass",
   "leftmarginkern",
+  "lefttwindemerits",
   "letcharcode",
   "letcsname",
   "letfrozen",
@@ -488,21 +571,30 @@ return {
   "letmathparent",
   "letmathspacing",
   "letprotected",
+  "lettolastnamedcs",
   "lettonothing",
-  "linebreakcriterium",
+  "linebreakchecks",
+  "linebreakoptional",
+  "linebreakpasses",
   "linedirection",
+  "localbreakpar",
   "localbrokenpenalty",
   "localcontrol",
   "localcontrolled",
+  "localcontrolledendless",
   "localcontrolledloop",
+  "localcontrolledrepeat",
   "localinterlinepenalty",
   "localleftbox",
   "localleftboxbox",
   "localmiddlebox",
   "localmiddleboxbox",
+  "localpretolerance",
   "localrightbox",
   "localrightboxbox",
+  "localtolerance",
   "lpcode",
+  "luaboundary",
   "luabytecode",
   "luabytecodecall",
   "luacopyinputnodes",
@@ -510,47 +602,46 @@ return {
   "luaescapestring",
   "luafunction",
   "luafunctioncall",
+  "luametatexmajorversion",
+  "luametatexminorversion",
+  "luametatexrelease",
   "luatexbanner",
   "luatexrevision",
   "luatexversion",
-  "mathaccent",
   "mathatom",
   "mathatomglue",
   "mathatomskip",
   "mathbackwardpenalties",
   "mathbeginclass",
-  "mathbinary",
+  "mathboundary",
+  "mathcharclass",
+  "mathcharfam",
+  "mathcharslot",
   "mathcheckfencesmode",
-  "mathclose",
+  "mathclass",
   "mathdictgroup",
+  "mathdictionary",
   "mathdictproperties",
   "mathdirection",
+  "mathdiscretionary",
   "mathdisplaymode",
+  "mathdisplaypenaltyfactor",
   "mathdisplayskipmode",
   "mathdoublescriptmode",
   "mathendclass",
   "matheqnogapstep",
-  "mathfenced",
   "mathfontcontrol",
   "mathforwardpenalties",
-  "mathfraction",
-  "mathghost",
   "mathgluemode",
   "mathgroupingmode",
-  "mathinner",
+  "mathinlinepenaltyfactor",
   "mathleftclass",
   "mathlimitsmode",
   "mathmainstyle",
-  "mathmiddle",
-  "mathnolimitsmode",
-  "mathopen",
-  "mathoperator",
-  "mathordinary",
-  "mathoverline",
+  "mathoptions",
+  "mathparentstyle",
   "mathpenaltiesmode",
-  "mathpunctuation",
-  "mathradical",
-  "mathrelation",
+  "mathpretolerance",
   "mathrightclass",
   "mathrulesfam",
   "mathrulesmode",
@@ -558,78 +649,149 @@ return {
   "mathscriptsmode",
   "mathslackmode",
   "mathspacingmode",
+  "mathstack",
   "mathstackstyle",
   "mathstyle",
   "mathstylefontid",
   "mathsurroundmode",
   "mathsurroundskip",
   "maththreshold",
-  "mathunderline",
+  "mathtolerance",
   "meaningasis",
   "meaningful",
   "meaningfull",
   "meaningles",
   "meaningless",
   "mugluespecdef",
+  "multiplyby",
   "mutable",
+  "mvlcurrentlyactive",
+  "nestedloopiterator",
   "noaligned",
+  "noarguments",
   "noatomruling",
   "noboundary",
   "nohrule",
+  "nomathchar",
+  "nooutputboxerror",
   "norelax",
   "normalizelinemode",
   "normalizeparmode",
+  "noscript",
   "nospaces",
+  "nosubprescript",
+  "nosubscript",
+  "nosuperprescript",
+  "nosuperscript",
+  "notexpanded",
   "novrule",
   "numericscale",
+  "numericscaled",
+  "numexperimental",
   "numexpression",
+  "optionalboundary",
   "orelse",
+  "orphanlinefactors",
   "orphanpenalties",
-  "orphanpenalty",
   "orunless",
   "outputbox",
   "overloaded",
   "overloadmode",
   "overshoot",
   "pageboundary",
+  "pageexcess",
   "pageextragoal",
+  "pagefistretch",
+  "pagelastdepth",
+  "pagelastfilllstretch",
+  "pagelastfillstretch",
+  "pagelastfilstretch",
+  "pagelastfistretch",
+  "pagelastheight",
+  "pagelastshrink",
+  "pagelaststretch",
   "pagevsize",
   "parametercount",
+  "parameterdef",
+  "parameterindex",
   "parametermark",
+  "parametermode",
   "parattribute",
   "pardirection",
+  "parfillleftskip",
+  "parfillrightskip",
+  "parinitleftskip",
+  "parinitrightskip",
+  "paroptions",
+  "parpasses",
+  "parpassesexception",
+  "parshapewidth",
   "permanent",
   "pettymuskip",
   "positdef",
   "postexhyphenchar",
   "posthyphenchar",
   "postinlinepenalty",
+  "postshortinlinepenalty",
   "prebinoppenalty",
+  "predisplaydirection",
   "predisplaygapfactor",
   "preexhyphenchar",
   "prehyphenchar",
   "preinlinepenalty",
   "prerelpenalty",
+  "preshortinlinepenalty",
+  "previousloopiterator",
+  "primescript",
+  "protecteddetokenize",
+  "protectedexpandeddetokenize",
   "protrudechars",
   "protrusionboundary",
   "pxdimen",
   "quitloop",
+  "quitloopnow",
   "quitvmode",
+  "rdivide",
+  "rdivideby",
+  "realign",
+  "resetlocalboxes",
   "resetmathspacing",
+  "restorecatcodetable",
+  "retained",
   "retokenized",
   "rightmarginkern",
+  "righttwindemerits",
   "rpcode",
   "savecatcodetable",
   "scaledemwidth",
   "scaledexheight",
   "scaledextraspace",
+  "scaledfontcharba",
+  "scaledfontchardp",
+  "scaledfontcharht",
+  "scaledfontcharic",
+  "scaledfontcharta",
+  "scaledfontcharwd",
   "scaledfontdimen",
+  "scaledfontemwidth",
+  "scaledfontexheight",
+  "scaledfontextraspace",
+  "scaledfontinterwordshrink",
+  "scaledfontinterwordspace",
+  "scaledfontinterwordstretch",
+  "scaledfontslantperpoint",
   "scaledinterwordshrink",
   "scaledinterwordspace",
   "scaledinterwordstretch",
+  "scaledmathaxis",
+  "scaledmathemwidth",
+  "scaledmathexheight",
   "scaledmathstyle",
   "scaledslantperpoint",
   "scantextokens",
+  "scriptspaceafterfactor",
+  "scriptspacebeforefactor",
+  "scriptspacebetweenfactor",
   "semiexpand",
   "semiexpanded",
   "semiprotected",
@@ -645,14 +807,36 @@ return {
   "setmathspacing",
   "shapingpenaltiesmode",
   "shapingpenalty",
+  "shortinlinemaththreshold",
+  "shortinlineorphanpenalty",
+  "showcodestack",
+  "showstack",
+  "singlelinepenalty",
   "snapshotpar",
+  "spacechar",
+  "spacefactormode",
+  "spacefactoroverload",
+  "spacefactorshrinklimit",
+  "spacefactorstretchlimit",
+  "specificationdef",
+  "splitextraheight",
+  "splitlastdepth",
+  "splitlastheight",
+  "splitlastshrink",
+  "splitlaststretch",
   "srule",
+  "subprescript",
+  "subscript",
+  "superprescript",
+  "superscript",
   "supmarkmode",
   "swapcsvalues",
   "tabsize",
   "textdirection",
   "thewithoutunit",
   "tinymuskip",
+  "tocharacter",
+  "toddlerpenalties",
   "todimension",
   "tohexadecimal",
   "tointeger",
@@ -660,6 +844,7 @@ return {
   "toksapp",
   "tokspre",
   "tolerant",
+  "tolimitedfloat",
   "tomathstyle",
   "toscaled",
   "tosparsedimension",
@@ -667,30 +852,48 @@ return {
   "tpack",
   "tracingadjusts",
   "tracingalignments",
+  "tracingbalancing",
   "tracingexpressions",
-  "tracingfonts",
+  "tracingfitness",
   "tracingfullboxes",
   "tracinghyphenation",
   "tracinginserts",
   "tracinglevels",
   "tracinglists",
+  "tracinglooseness",
   "tracingmarks",
   "tracingmath",
+  "tracingmvl",
   "tracingnodes",
+  "tracingorphans",
+  "tracingpasses",
   "tracingpenalties",
+  "tracingtoddlers",
   "tsplit",
   "uleaders",
   "undent",
+  "unexpandedendless",
   "unexpandedloop",
+  "unexpandedrepeat",
   "unhpack",
   "unletfrozen",
   "unletprotected",
   "untraced",
   "unvpack",
   "variablefam",
+  "vbadnessmode",
+  "vbalance",
+  "vbalancedbox",
+  "vbalanceddeinsert",
+  "vbalanceddiscard",
+  "vbalancedinsert",
+  "vbalancedreinsert",
+  "vbalancedtop",
   "virtualhrule",
   "virtualvrule",
   "vpack",
+  "vpenalty",
+  "vsplitchecks",
   "wordboundary",
   "wrapuppar",
   "xdefcsname",
@@ -829,7 +1032,6 @@ return {
   "accent",
   "adjdemerits",
   "advance",
-  "advanceby",
   "afterassignment",
   "aftergroup",
   "atop",
@@ -874,12 +1076,12 @@ return {
   "displaywidowpenalty",
   "displaywidth",
   "divide",
-  "divideby",
   "doublehyphendemerits",
   "dp",
   "dump",
   "edef",
   "else",
+  "emergencyextrastretch",
   "emergencystretch",
   "end",
   "endcsname",
@@ -923,6 +1125,7 @@ return {
   "hfill",
   "hfilneg",
   "hfuzz",
+  "hkern",
   "holdinginserts",
   "hrule",
   "hsize",
@@ -978,12 +1181,16 @@ return {
   "lower",
   "lowercase",
   "mark",
+  "mathaccent",
   "mathbin",
   "mathchar",
   "mathchardef",
   "mathchoice",
+  "mathclose",
   "mathcode",
+  "mathinner",
   "mathop",
+  "mathopen",
   "mathord",
   "mathpunct",
   "mathrel",
@@ -1000,7 +1207,6 @@ return {
   "moveright",
   "mskip",
   "multiply",
-  "multiplyby",
   "muskip",
   "muskipdef",
   "newlinechar",
@@ -1031,11 +1237,8 @@ return {
   "pagestretch",
   "pagetotal",
   "par",
-  "parfillleftskip",
   "parfillskip",
   "parindent",
-  "parinitleftskip",
-  "parinitrightskip",
   "parshape",
   "parskip",
   "patterns",
@@ -1096,6 +1299,7 @@ return {
   "topmark",
   "topskip",
   "tracingcommands",
+  "tracingloners",
   "tracinglostchars",
   "tracingmacros",
   "tracingonline",
@@ -1125,6 +1329,7 @@ return {
   "vfill",
   "vfilneg",
   "vfuzz",
+  "vkern",
   "vrule",
   "vsize",
   "vskip",
