@@ -45,13 +45,12 @@ local nuts              = nodes.nuts
 
 local getid             = nuts.getid
 local getsubtype        = nuts.getsubtype
-local getdisc           = nuts.getdisc
+local getreplace        = nuts.getreplace
 local getnext           = nuts.getnext
 local getprev           = nuts.getprev
 local getboth           = nuts.getboth
 local getdata           = nuts.getdata
 local getglyphdata      = nuts.getglyphdata
------ getdisc           = nuts.getdisc
 
 local setchar           = nuts.setchar
 local setlink           = nuts.setlink
@@ -61,21 +60,19 @@ local setprev           = nuts.setprev
 local isglyph           = nuts.isglyph -- unchecked
 local ischar            = nuts.ischar  -- checked
 
------ traverse_id       = nuts.traverse_id
------ traverse_char     = nuts.traverse_char
 local nextboundary      = nuts.traversers.boundary
 local nextdisc          = nuts.traversers.disc
 local nextchar          = nuts.traversers.char
 
-local flush_node        = nuts.flush
+local flushnode         = nuts.flush
 
 local disc_code         = nodecodes.disc
 local boundary_code     = nodecodes.boundary
 
 local wordboundary_code = boundarycodes.word
 
-local protect_glyphs    = nuts.protect_glyphs
-local unprotect_glyphs  = nuts.unprotect_glyphs
+local protectglyphs     = nuts.protectglyphs
+local unprotectglyphs   = nuts.unprotectglyphs
 
 local setmetatableindex = table.setmetatableindex
 
@@ -139,37 +136,14 @@ fonts.hashes.processes   = fontprocesses
 -- inside a run which means that we need to keep track of this which in turn complicates matters
 -- in a way i don't like
 
--- we need to deal with the basemode fonts here and can only run over ranges as we
--- otherwise get luatex craches due to all kind of asserts in the disc/lig builder
+-- we need to deal with the basemode fonts here and can only run over ranges as we otherwise get
+-- luatex craches due to all kind of asserts in the disc/lig builder
+
+-- there is no gain in merging used (dynamic 0) and dynamics apart from a bit less code
 
 local ligaturing = nuts.ligaturing
 local kerning    = nuts.kerning
 
--- -- -- this will go away
---
--- local disccodes          = nodes.disccodes
--- local explicitdisc_code  = disccodes.explicit
--- local automaticdisc_code = disccodes.automatic
--- local expanders          = nil
---
--- function fonts.setdiscexpansion(v)
---     if v == nil or v == true then
---         expanders = languages and languages.expanders
---     elseif type(v) == "table" then
---         expanders = v
---     else
---         expanders = false
---     end
--- end
---
--- function fonts.getdiscexpansion()
---     return expanders and true or false
--- end
---
--- fonts.setdiscexpansion(true)
---
--- -- -- till here
-
 local function start_trace(head)
     run = run + 1
     report_fonts()
@@ -193,19 +167,15 @@ local function start_trace(head)
     end
 end
 
-local function stop_trace(u,usedfonts,a,attrfonts,b,basefonts,r,redundant,e,expanders)
+local function stop_trace(u,usedfonts,a,attrfonts,b,basefonts,r,redundant)
     report_fonts()
     report_fonts("statics : %s",u > 0 and concat(keys(usedfonts)," ") or "none")
     report_fonts("dynamics: %s",a > 0 and concat(keys(attrfonts)," ") or "none")
     report_fonts("built-in: %s",b > 0 and b or "none")
     report_fonts("removed : %s",r > 0 and r or "none")
- -- if expanders then
- --     report_fonts("expanded: %s",e > 0 and e or "none")
- -- end
     report_fonts()
 end
 
-
 do
 
     local usedfonts
@@ -221,10 +191,10 @@ do
     local lastproc
     local lastnone
 
-    local a, u, b, r, e
+    local a, u, b, r
 
     local function protectnone()
-        protect_glyphs(firstnone,lastnone)
+        protectglyphs(firstnone,lastnone)
         firstnone = nil
     end
 
@@ -307,7 +277,7 @@ do
         lastproc  = nil
         lastnone  = nil
 
-        a, u, b, r, e = 0, 0, 0, 0, 0
+        a, u, b, r = 0, 0, 0, 0
 
         if trace_fontrun then
             start_trace(head)
@@ -420,20 +390,18 @@ do
                         end
                     end
                 end
-                flush_node(r)
+                flushnode(r)
             end
         end
 
         if force_discrun then
-
             -- basefont is not supported in disc only runs ... it would mean a lot of
-            -- ranges .. we could try to run basemode as a separate processor run but
-            -- not for now (we can consider it when the new node code is tested
+            -- ranges .. we could try to run basemode as a separate processor run but not
+            -- for now (we can consider it when the new node code is tested
             for d in nextdisc, head do
-                -- we could use first_glyph, only doing replace is good enough because
-                -- pre and post are normally used for hyphens and these come from fonts
-                -- that part of the hyphenated word
-                local _, _, r = getdisc(d)
+                -- doing only replace is good enough because pre and post are normally used
+                -- for hyphens and these come from fonts that part of the hyphenated word
+                local r = getreplace(d)
                 if r then
                     local prevfont = nil
                     local prevattr = nil
@@ -465,19 +433,13 @@ do
                     if firstnone then
                         protectnone()
                     end
-             -- elseif expanders then
-             --     local subtype = getsubtype(d)
-             --     if subtype == automaticdisc_code or subtype == explicitdisc_code then
-             --         expanders[subtype](d)
-             --         e = e + 1
-             --     end
                 end
             end
 
         end
 
         if trace_fontrun then
-            stop_trace(u,usedfonts,a,attrfonts,b,basefonts,r,redundant,e,expanders)
+            stop_trace(u,usedfonts,a,attrfonts,b,basefonts,r,redundant)
         end
 
         -- in context we always have at least 2 processors
@@ -493,7 +455,7 @@ do
             local attr = a > 0 and 0 or false -- 0 is the savest way
             for font, processors in next, usedfonts do -- unordered
                 for i=1,#processors do
-                    head = processors[i](head,font,attr,direction,u)
+                    head = processors[i](head,font,attr,direction)
                 end
             end
         end
@@ -510,7 +472,7 @@ do
             for font, dynamics in next, attrfonts do
                 for attribute, processors in next, dynamics do -- unordered, attr can switch in between
                     for i=1,#processors do
-                        head = processors[i](head,font,attribute,direction,a)
+                        head = processors[i](head,font,attribute,direction)
                     end
                 end
             end
@@ -578,5 +540,5 @@ do
 
 end
 
-handlers.protectglyphs   = protect_glyphs
-handlers.unprotectglyphs = unprotect_glyphs
+handlers.protectglyphs   = protectglyphs
+handlers.unprotectglyphs = unprotectglyphs
