@@ -1,5 +1,6 @@
 if not modules then modules = { } end modules ['spac-ali'] = {
     version   = 1.001,
+    optimize  = true,
     comment   = "companion to spac-ali.mkiv",
     author    = "Hans Hagen, PRAGMA-ADE, Hasselt NL",
     copyright = "PRAGMA ADE / ConTeXt Development Team",
@@ -24,11 +25,14 @@ local getid            = nuts.getid
 local getlist          = nuts.getlist
 local setlist          = nuts.setlist
 local setlink          = nuts.setlink
+local getdirection     = nuts.getdirection
 local takeattr         = nuts.takeattr
 local getsubtype       = nuts.getsubtype
 local getwidth         = nuts.getwidth
 local findtail         = nuts.tail
 
+local righttoleft_code = nodes.dirvalues.righttoleft
+
 local hpack_nodes      = nuts.hpack
 
 local unsetvalue       = attributes.unsetvalue
@@ -54,8 +58,8 @@ typesetters            = typesetters or { }
 local alignments       = { }
 typesetters.alignments = alignments
 
-local report_realign   = logs.reporter("typesetters","margindata")
-local trace_realign    = trackers.register("typesetters.margindata", function(v) trace_margindata = v end)
+local report_realign   = logs.reporter("typesetters","realign")
+local trace_realign    = trackers.register("typesetters.realign", function(v) trace_realign = v end)
 
 local nofrealigned     = 0
 
@@ -64,7 +68,7 @@ local nofrealigned     = 0
 -- raggedright     0           0         fil
 -- raggedcenter    0 +         0 +        -
 
-local function handler(head,leftpage,realpageno) -- traverse_list
+local function handler(head,leftpage,realpageno) -- traverselist
     local current = head
     while current do
         local id = getid(current)
@@ -85,19 +89,30 @@ local function handler(head,leftpage,realpageno) -- traverse_list
                         elseif align == 2 then -- flushleft
                             action = leftpage and 2 or 1
                         end
+                        -- WS: watch this
+                        local direction = getdirection(current)
+                        -- or should this happen at the tex end:
+                        if direction == righttoleft_code then
+                            if action == 1 then
+                                action = 2
+                            elseif action == 2 then
+                                action = 1
+                            end
+                        end
+                        --
                         if action == 1 then
                             local head = getlist(current)
                             setlink(findtail(head),new_stretch(3)) -- append
-                            setlist(current,hpack_nodes(head,getwidth(current),"exactly"))
+                            setlist(current,hpack_nodes(head,getwidth(current),"exactly",direction))
                             if trace_realign then
                                 report_realign("flushing left, align %a, page %a, realpage %a",align,pageno,realpageno)
                             end
                         elseif action == 2 then
                             local list = getlist(current)
                             local head = setlink(new_stretch(3),list) -- prepend
-                            setlist(current,hpack_nodes(head,getwidth(current),"exactly"))
+                            setlist(current,hpack_nodes(head,getwidth(current),"exactly",direction))
                             if trace_realign then
-                                report_realign("flushing right. align %a, page %a, realpage %a",align,pageno,realpageno)
+                                report_realign("flushing right, align %a, page %a, realpage %a",align,pageno,realpageno)
                             end
                         elseif trace_realign then
                             report_realign("invalid flushing, align %a, page %a, realpage %a",align,pageno,realpageno)
