@@ -35,6 +35,8 @@ local stopapplyprocessor  = processors.stopapply
 local texsetcount         = tex.setcount
 local texgetcount         = tex.getcount
 
+local texconditionals     = tex.conditionals
+
 local ctx_convertnumber   = context.convertnumber
 
 -- storage
@@ -74,7 +76,8 @@ function pages.save(prefixdata,numberdata,extradata)
             report_pages("saving page %s.%s",realpage,userpage)
         end
         local viewerprefix = extradata.viewerprefix
-        local state = extradata.state
+        local state        = extradata.state
+        local label        = extradata.label
         local data = {
             number       = userpage,
             viewerprefix = viewerprefix ~= "" and viewerprefix or nil,
@@ -82,6 +85,8 @@ function pages.save(prefixdata,numberdata,extradata)
             block        = sections.currentblock(),
             prefixdata   = prefixdata and helpers.simplify(prefixdata),
             numberdata   = numberdata and helpers.simplify(numberdata),
+            marked       = pages.markedlist(realpage), -- not yet defined
+            label        = label and label ~= "" and label or nil,
         }
         tobesaved[realpage] = data
         if not collected[realpage] then
@@ -144,7 +149,7 @@ function pages.number(realdata,pagespec)
         stopapplyprocessor()
     end
     if stopper ~= "" then
-        applyprocessors(stopper)
+        applyprocessor(stopper)
     end
 end
 
@@ -252,7 +257,7 @@ function helpers.analyze(entry,specification)
     if not section then
         return entry, false, "no section"
     end
-    local sectiondata = sections.collected[references.section]
+    local sectiondata = references.sectiondata or sections.collected[references.section] -- so we use an already resolved external one
     if not sectiondata then
         return entry, false, "no section data"
     end
@@ -269,7 +274,7 @@ function helpers.analyze(entry,specification)
     return entry, sectiondata, "okay"
 end
 
-function helpers.prefix(data,prefixspec,nosuffix)
+function helpers.prefix(data,prefixspec,nosuffix) -- not only page
     if data then
         local _, prefixdata, status = helpers.analyze(data,prefixspec)
         if prefixdata then
@@ -380,6 +385,33 @@ function sections.prefixedconverted(name,prefixspec,numberspec)
     end
 end
 
+function pages.getlabels()
+    local pages  = structures.pages.tobesaved
+    local labels = false
+    for i=1,#pages do
+        local p = pages[i]
+        if p then
+            local label = p.label
+            if label and label ~= "" then
+                if not labels then
+                    labels = { }
+                end
+                local l = labels[label]
+                local t = type(l)
+                if t == "number" then
+                    l = { l, i }
+                elseif t == "table" then
+                    l[#l+1] = i
+                else
+                    l = i
+                end
+                labels[label] = l
+            end
+        end
+    end
+    return labels
+end
+
 --
 
 implement {
@@ -404,6 +436,7 @@ implement {
         {
             { "viewerprefix" },
             { "state" },
+            { "label" },
         }
     }
 }
@@ -440,6 +473,6 @@ implement { -- weird place
 
 interfaces.implement {
     name      = "pageofinternal",
-    arguments = { "integer" },
+    arguments = "integer",
     actions   = helpers.pageofinternal,
 }
