@@ -13,7 +13,7 @@ local tables     = utilities.tables
 local format, gmatch, gsub, sub = string.format, string.gmatch, string.gsub, string.sub
 local concat, insert, remove, sort = table.concat, table.insert, table.remove, table.sort
 local setmetatable, getmetatable, tonumber, tostring, rawget = setmetatable, getmetatable, tonumber, tostring, rawget
-local type, next, rawset, tonumber, tostring, load, select = type, next, rawset, tonumber, tostring, load, select
+local type, next, rawset, rawget, tonumber, tostring, load, select = type, next, rawset, rawget, tonumber, tostring, load, select
 local lpegmatch, P, Cs, Cc = lpeg.match, lpeg.P, lpeg.Cs, lpeg.Cc
 local sortedkeys, sortedpairs = table.sortedkeys, table.sortedpairs
 local formatters = string.formatters
@@ -225,11 +225,11 @@ local function toxml(t,d,result,step)
         if tv == "table" then
             if tk == "number" then
                 r = r + 1 result[r] = formatters["%s<entry n='%s'>"](s,k)
-                toxml(v,d+step,result,step)
+                r = toxml(v,d+step,result,step)
                 r = r + 1 result[r] = formatters["%s</entry>"](s,k)
             else
                 r = r + 1 result[r] = formatters["%s<%s>"](s,k)
-                toxml(v,d+step,result,step)
+                r = toxml(v,d+step,result,step)
                 r = r + 1 result[r] = formatters["%s</%s>"](s,k)
             end
         elseif tv == "string" then
@@ -244,6 +244,7 @@ local function toxml(t,d,result,step)
             r = r + 1 result[r] = formatters["%s<%s>%S</%s>"](s,k,v,k)
         end
     end
+    return r
 end
 
 -- function table.toxml(t,name,nobanner,indent,spaces)
@@ -417,9 +418,9 @@ if JITSUPPORTED then
 
 else
 
-    local f_v = formatters["[%q]=%q,"]
-    local f_t = formatters["[%q]="]
-    local f_q = formatters["%q,"]
+ -- local f_v = formatters["[%q]=%q,"]
+ -- local f_t = formatters["[%q]="]
+ -- local f_q = formatters["%q,"]
 
     function table.fastserialize(t,prefix) -- todo, move local function out
         local r = { type(prefix) == "string" and prefix or "return" }
@@ -432,7 +433,7 @@ else
                 local v = t[0]
                 if v then
                     m = m + 1
-                    r[m] = "[0]='"
+                    r[m] = "[0]="
                     if type(v) == "table" then
                         fastserialize(v)
                     else
@@ -720,6 +721,7 @@ local function serialize(root,name,specification)
 
     local t    -- = { }
     local n       = 1
+ -- local m       = 0 -- no gain
     local unknown = false
 
     local function do_serialize(root,name,depth,level,indexed)
@@ -850,6 +852,12 @@ local function serialize(root,name,specification)
                         n = n + 1 t[n] = f_key_str_value_str(depth,tostring(k),tostring(v))
                     end
                 end
+             -- if n > 100000 then -- no gain
+             --     local k = m + 1
+             --     t[k] = concat(t,"\n",k,n)
+             --     n = k
+             --     m = k
+             -- end
             end
         end
         if level > 0 then
@@ -898,6 +906,7 @@ local function serialize(root,name,specification)
     n = n + 1
     t[n] = f_table_finish()
     return concat(t,"\n")
+ -- return concat(t,"\n",1,n) -- no gain
 end
 
 table.serialize = serialize
@@ -961,3 +970,85 @@ function table.ordered(t)
         return function() end
     end
 end
+
+-- function table.randomremove(t,n)
+--     if not n then
+--         n = #t
+--     end
+--     if n > 0 then
+--         return remove(t,random(1,n))
+--     end
+-- end
+
+local function combine(target,source)
+    -- no copy so if that is needed one needs to deepcopy source first
+    if target then
+        for k, v in next, source do
+            if type(v) == "table" then
+               target[k] = combine(target[k],source[k])
+            else
+               target[k] = v
+            end
+        end
+        return target
+    else
+        return source
+    end
+end
+
+table.combine = combine
+
+-- If needed we can add something (some discussion on the list but I'm not sure if
+-- it makes sense because merging such mixed tables is quite unusual.
+--
+-- function table.himerged(...)
+--     local result = { }
+--     local r      = 0
+--     for i=1,select("#",...) do
+--         local s = select(i,...)
+--         if s then
+--             for k, v in next, s do
+--                 if type(k) == "number"  then
+--                     r = r + 1
+--                     result[r] = v
+--                 else
+--                     result[k] = v
+--                 end
+--             end
+--         end
+--     end
+--     return result
+-- end
+
+-- can be a helper
+
+-- function table.identify(t)
+--     local size = #t
+--     if size > 0 then
+--         local count  = 0
+--         local simple = true
+--         for k, v in next, t do
+--             if type(k) ~= "number" then
+--                 count = -1
+--                 break
+--             end
+--             count = count + 1
+--             if simple and type(v) == "table" then
+--                 simple = false
+--             end
+--         end
+--         if rawget(t,0) then
+--             return
+--                 (size == count - 1) and size,
+--                 true,
+--                 simple
+--         else
+--             return
+--                 (size == count) and size,
+--                 false,
+--                 simple
+--         end
+--     else
+--         return false, false, false
+--     end
+-- end
