@@ -976,7 +976,7 @@ end
 local collected_digits   = { }
 local collected_filename = "asciimath-digits.lua"
 
-function numbermess(s)
+local function numbermess(s)
     if splitmethod then
         local d = lpegmatch(splitmethod,s,1,digitseparator,digitsymbol)
         if not d and symbolmethod then
@@ -1597,7 +1597,7 @@ local function collapse_infixes_1(t)
     return t
 end
 
-function collapse_limits(t)
+local function collapse_limits(t)
     local n, m, i = #t, 0, 1
     while i <= n do
         m = m + 1
@@ -1937,7 +1937,7 @@ local function wrapup(collected,indexed)
     end
 end
 
-function collect(fpattern,element,collected,indexed)
+local function collect(fpattern,element,collected,indexed)
     local element   = element or "am"
     local mpattern  = formatters["<%s>(.-)</%s>"](element,element)
     local filenames = resolvers.findtexfile(fpattern)
@@ -1971,7 +1971,7 @@ function collect(fpattern,element,collected,indexed)
     return collected, indexed
 end
 
-function filter(root,pattern,collected,indexed)
+local function filter(root,pattern,collected,indexed)
     if not pattern or pattern == "" then
         pattern = "am"
     end
